// ************************************************************************
// **   FILENAME:          FAXLIB.CPP
// **
// **   CREATED:           2/8/96
// **   LAST MODIFIED:     2/8/96
// **
// **   VERSION:           1.0
// **   MODULE:            FAXLIB.CPP
// **
// **   DESCRIPTION:       Dialogic Fax Library Compatability Module   
// **
// **   Copyright (c) 1996 Dialogic Corp.
// ************************************************************************

#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <io.h>
#include <sys\stat.h>
#include "srllib.h"
#include "dxxxlib.h"
#include "faxlib.h"

// prototypes for 'not implemented' functions
static long fx_long_notimp (int);
static int fx_int_notimp (int);
static DF_IOTT *fx_DF_IOTT_notimp (int);
static void fx_void_notimp (int);
static char *fx_null_notimp (int);

long fx_GetDLLVersion (LPDWORD, LPDWORD);

// typedefs and #defines
typedef void (*func_ptr)(void);

struct func_table
{
  char *fname;
  func_ptr fptr;
};

typedef enum _FUNCPTR_IDS
{
  ATFX_BADIOTT_ID = 0,
  ATFX_BADPAGE_ID,
  ATFX_BADSCANLINES_ID,
  ATFX_BSTAT_ID,
  ATFX_CHTYPE_ID,
  ATFX_CODING_ID,
  ATFX_ECM_ID,
  ATFX_ESTAT_ID,
  ATFX_FXVERSION_ID,
  ATFX_LASTIOTT_ID,
  ATFX_PGXFER_ID,
  ATFX_PHDCMD_ID,
  ATFX_PHDRPY_ID,
  ATFX_RESLN_ID,
  ATFX_RTNPAGES_ID,
  ATFX_SCANLINES_ID,
  ATFX_SPEED_ID,
  ATFX_STATE_ID,
  ATFX_TERMMSK_ID,
  ATFX_TFBADTAG_ID,
  ATFX_TFNOTAG_ID,
  ATFX_TFPGBASE_ID,
  ATFX_TRCOUNT_ID,
  ATFX_WIDTH_ID,
  fx_close_ID,
  fx_getDCS_ID,
  fx_getDIS_ID,
  fx_getNSF_ID,
  fx_getparm_ID,
  fx_getxmitslot_ID,
  fx_initstat_ID,
  fx_listen_ID,
  fx_open_ID,
  fx_rcvfax_ID,
  fx_rcvfax2_ID,
  fx_route_ID,
  fx_sendfax_ID,
  fx_setiott_ID,
  fx_setparm_ID,
  fx_setuio_ID,
  fx_stopch_ID,
  fx_unlisten_ID,
  fx_loadfont_ID,
  fx_rtvContinue_ID,
  fx_originate_ID
} FUNCPTR_IDS;

typedef DF_IOTT * (*ptr_ATFX_BADIOTT) (int);
typedef long      (*ptr_ATFX_BADPAGE) (int);
typedef long      (*ptr_ATFX_BADSCANLINES) (int);
typedef long      (*ptr_ATFX_BSTAT) (int);
typedef long      (*ptr_ATFX_CHTYPE) (int);
typedef long      (*ptr_ATFX_CODING) (int);
typedef long      (*ptr_ATFX_ECM) (int);
typedef long      (*ptr_ATFX_ESTAT) (int);
typedef const char *    (*ptr_ATFX_FXVERSION) (int);
typedef DF_IOTT * (*ptr_ATFX_LASTIOTT) (int);
typedef long      (*ptr_ATFX_PGXFER) (int);
typedef long      (*ptr_ATFX_PHDCMD) (int);
typedef long      (*ptr_ATFX_PHDRPY) (int);
typedef long      (*ptr_ATFX_RESLN) (int);
typedef long      (*ptr_ATFX_RTNPAGES) (int);
typedef long      (*ptr_ATFX_SCANLINES) (int);
typedef long      (*ptr_ATFX_SPEED) (int);
typedef long      (*ptr_ATFX_STATE) (int);
typedef long      (*ptr_ATFX_TERMMSK) (int);
typedef long      (*ptr_ATFX_TFBADTAG) (int);
typedef long      (*ptr_ATFX_TFNOTAG) (int);
typedef long      (*ptr_ATFX_TFPGBASE) (int);
typedef long      (*ptr_ATFX_TRCOUNT) (int);
typedef long      (*ptr_ATFX_WIDTH) (int);
typedef int       (*ptr_fx_close) (int);
typedef int       (*ptr_fx_getDCS) (int, DF_DCS *);
typedef int       (*ptr_fx_getDIS) (int, DF_DIS *);
typedef int       (*ptr_fx_getNSF) (int, unsigned short, char *);
typedef int       (*ptr_fx_getparm) (int, unsigned long, void *);
typedef int       (*ptr_fx_getxmitslot) (int, SC_TSINFO *);
typedef int       (*ptr_fx_initstat) (int, int);
typedef int       (*ptr_fx_listen) (int, SC_TSINFO *);
typedef int       (*ptr_fx_open) (const char *, int);
typedef int       (*ptr_fx_rcvfax) (int, const char *, unsigned long);
typedef int       (*ptr_fx_rcvfax2) (int, int, unsigned long);
typedef int       (*ptr_fx_route) (int, int, int);
typedef int       (*ptr_fx_sendfax) (int, DF_IOTT *, unsigned long);
typedef void      (*ptr_fx_setiott) (DF_IOTT *, int, unsigned short, unsigned short);
typedef int       (*ptr_fx_setparm) (int, unsigned long, const void *);
typedef void      (*ptr_fx_setuio) (DF_UIO);
typedef int       (*ptr_fx_stopch) (int, int);
typedef int       (*ptr_fx_unlisten) (int);
typedef int       (*ptr_fx_loadfont) (int, int, unsigned long);
typedef int       (*ptr_fx_rtvContinue) (int, int);
typedef int		  (*ptr_fx_originate)(int, int);
// Global variables
static struct func_table FaxFunc[] =
{
  {"ATFX_BADIOTT", (func_ptr)fx_DF_IOTT_notimp},
  {"ATFX_BADPAGE", (func_ptr)fx_long_notimp},
  {"ATFX_BADSCANLINES", (func_ptr)fx_long_notimp},
  {"ATFX_BSTAT", (func_ptr)fx_long_notimp},
  {"ATFX_CHTYPE", (func_ptr)fx_long_notimp},
  {"ATFX_CODING", (func_ptr)fx_long_notimp},
  {"ATFX_ECM", (func_ptr)fx_long_notimp},
  {"ATFX_ESTAT", (func_ptr)fx_long_notimp},
  {"ATFX_FXVERSION", (func_ptr)fx_null_notimp},
  {"ATFX_LASTIOTT", (func_ptr)fx_DF_IOTT_notimp},
  {"ATFX_PGXFER", (func_ptr)fx_long_notimp},
  {"ATFX_PHDCMD", (func_ptr)fx_long_notimp},
  {"ATFX_PHDRPY", (func_ptr)fx_long_notimp},
  {"ATFX_RESLN", (func_ptr)fx_long_notimp},
  {"ATFX_RTNPAGES", (func_ptr)fx_long_notimp},
  {"ATFX_SCANLINES", (func_ptr)fx_long_notimp},
  {"ATFX_SPEED", (func_ptr)fx_long_notimp},
  {"ATFX_STATE", (func_ptr)fx_long_notimp},
  {"ATFX_TERMMSK", (func_ptr)fx_long_notimp},
  {"ATFX_TFBADTAG", (func_ptr)fx_long_notimp},
  {"ATFX_TFNOTAG", (func_ptr)fx_long_notimp},
  {"ATFX_TFPGBASE", (func_ptr)fx_long_notimp},
  {"ATFX_TRCOUNT", (func_ptr)fx_long_notimp},
  {"ATFX_WIDTH", (func_ptr)fx_long_notimp},
  {"fx_close", (func_ptr)fx_int_notimp},
  {"fx_getDCS", (func_ptr)fx_int_notimp},
  {"fx_getDIS", (func_ptr)fx_int_notimp},
  {"fx_getNSF", (func_ptr)fx_int_notimp},
  {"fx_getparm", (func_ptr)fx_int_notimp},
  {"fx_getxmitslot", (func_ptr)fx_int_notimp},
  {"fx_initstat", (func_ptr)fx_int_notimp},
  {"fx_listen", (func_ptr)fx_int_notimp},
  {"fx_open", (func_ptr)fx_int_notimp},
  {"fx_rcvfax", (func_ptr)fx_int_notimp},
  {"fx_rcvfax2", (func_ptr)fx_int_notimp},
  {"fx_route", (func_ptr)fx_int_notimp},
  {"fx_sendfax", (func_ptr)fx_int_notimp},
  {"fx_setiott", (func_ptr)fx_void_notimp},
  {"fx_setparm", (func_ptr)fx_int_notimp},
  {"fx_setuio", (func_ptr)fx_void_notimp},
  {"fx_stopch", (func_ptr)fx_int_notimp},
  {"fx_unlisten", (func_ptr)fx_int_notimp},
  {"fx_loadfont", (func_ptr)fx_int_notimp},
  {"fx_rtvContinue", (func_ptr)fx_int_notimp},
  {"fx_originate", (func_ptr)fx_int_notimp}
};

char caMTLibName[] = "LIBFAXMT.DLL";
char caSTLibName[] = "LIBFAX.DLL";
char *pcLibName;

long fx_libinit (unsigned short usFlag)
{
  struct func_table *pFax = FaxFunc;
  func_ptr pProcAddress;
  static HINSTANCE hLibrary = NULL;
  DWORD dwNumberOfFunctions;
  DWORD dwFileVer, dwProdVer;
  // char caTemp[80];

  // Check if it's already open
  if (hLibrary != (HINSTANCE)NULL)
  {
    return -1;
  }

  // Determine which library to load
  if ((usFlag & DLGC_MT) == DLGC_MT)
  {
    pcLibName = caMTLibName;
  }
  else
  {
    pcLibName = caMTLibName;
    fx_GetDLLVersion (&dwFileVer, &dwProdVer);
    if (dwFileVer <= 0x0004000A)
    {
      pcLibName = caSTLibName;
    }
  }

  dwNumberOfFunctions = sizeof (FaxFunc) / sizeof (FaxFunc[0]);

  // Try to load the library
  hLibrary = LoadLibrary (pcLibName);
  if (hLibrary == (HINSTANCE)NULL)
  {
    return 0;
  }

  // Set up each entry to a default not implemented
  // function and then overwrite the true address if the 
  // function is actually present in the DLL
  while (dwNumberOfFunctions--)
  {
    pProcAddress = (func_ptr)GetProcAddress (hLibrary, pFax->fname);
    if (pProcAddress)
    {
      pFax->fptr = pProcAddress;
    }
    else
    {
      //sprintf (caTemp, "Function %s not implemented, error %X.", pFax->fname, GetLastError());
      //MessageBox (HWND_DESKTOP, caTemp, "CCLIB", MB_ICONINFORMATION | MB_OK);
    }
    pFax++;
  }

  return 0;
}

long fx_GetDLLVersion (LPDWORD pdwFileVer, LPDWORD pdwProdVer)
{
  long lTemp;
  lTemp = sr_GetVersion (pcLibName, pdwFileVer, pdwProdVer);
  if (*pdwFileVer == 0x00030032)
  {
    *pdwFileVer = 0x0004000B;
  }
  return lTemp;
}

int fx_set_lasterr (int iErr, long lError)
{
  // the real _fx_set_lasterr in FXMAIN.C relies on global data not
  // exported outside of the LIBFAX.DLL.  And dx_set_lasterr() is not
  // is not in DXXXLIB.H and is not exported from that module...
  dx_set_lasterr (iErr, lError);
  return 0;
}

long fx_long_notimp (int iErr)
{
  fx_set_lasterr (iErr, EFX_NOTIMP);
  return -1;
}

int fx_int_notimp (int iErr)
{
  fx_set_lasterr (iErr, EFX_NOTIMP);
  return -1;
}

DF_IOTT *fx_DF_IOTT_notimp (int iErr)
{
  fx_set_lasterr (iErr, EFX_NOTIMP);
  return (DF_IOTT *) NULL;
}

void fx_void_notimp (int iErr)
{
  fx_set_lasterr (iErr, EFX_NOTIMP);
  return;
}

char *fx_null_notimp (int iErr)
{
  fx_set_lasterr (iErr, EFX_NOTIMP);
  return (char *) NULL;
}

int fx_dflt_errno (void)
{
  return errno;
}





DF_IOTT *ATFX_BADIOTT (int iChannel)
{
  ptr_ATFX_BADIOTT p;

  p = (ptr_ATFX_BADIOTT) *FaxFunc[ATFX_BADIOTT_ID].fptr;
  return ( (*p) (iChannel));
}

long ATFX_BADPAGE (int iChannel)
{
  ptr_ATFX_BADPAGE p;

  p = (ptr_ATFX_BADPAGE) *FaxFunc[ATFX_BADPAGE_ID].fptr;
  return ( (*p) (iChannel));
}

long ATFX_BADSCANLINES (int iChannel)
{
  ptr_ATFX_BADSCANLINES p;

  p = (ptr_ATFX_BADSCANLINES) *FaxFunc[ATFX_BADSCANLINES_ID].fptr;
  return ( (*p) (iChannel));
}

long ATFX_BSTAT (int iChannel)
{
  ptr_ATFX_BSTAT p;

  p = (ptr_ATFX_BSTAT) *FaxFunc[ATFX_BSTAT_ID].fptr;
  return ( (*p) (iChannel));
}

long ATFX_CHTYPE (int iChannel)
{
  ptr_ATFX_CHTYPE p;

  p = (ptr_ATFX_CHTYPE) *FaxFunc[ATFX_CHTYPE_ID].fptr;
  return ( (*p) (iChannel));
}

long ATFX_CODING (int iChannel)
{
  ptr_ATFX_CODING p;

  p = (ptr_ATFX_CODING) *FaxFunc[ATFX_CODING_ID].fptr;
  return ( (*p) (iChannel));
}

long ATFX_ECM (int iChannel)
{
  ptr_ATFX_ECM p;

  p = (ptr_ATFX_ECM) *FaxFunc[ATFX_ECM_ID].fptr;
  return ( (*p) (iChannel));
}

long ATFX_ESTAT (int iChannel)
{
  ptr_ATFX_ESTAT p;

  p = (ptr_ATFX_ESTAT) *FaxFunc[ATFX_ESTAT_ID].fptr;
  return ( (*p) (iChannel));
}

const char * ATFX_FXVERSION (int iChannel)
{
  ptr_ATFX_FXVERSION p;

  p = (ptr_ATFX_FXVERSION) *FaxFunc[ATFX_FXVERSION_ID].fptr;
  return ( (*p) (iChannel));
}

DF_IOTT *ATFX_LASTIOTT (int iChannel)
{
  ptr_ATFX_LASTIOTT p;

  p = (ptr_ATFX_LASTIOTT) *FaxFunc[ATFX_LASTIOTT_ID].fptr;
  return ( (*p) (iChannel));
}

long ATFX_PGXFER (int iChannel)
{
  ptr_ATFX_PGXFER p;

  p = (ptr_ATFX_PGXFER) *FaxFunc[ATFX_PGXFER_ID].fptr;
  return ( (*p) (iChannel));
}

long ATFX_PHDCMD (int iChannel)
{
  ptr_ATFX_PHDCMD p;

  p = (ptr_ATFX_PHDCMD) *FaxFunc[ATFX_PHDCMD_ID].fptr;
  return ( (*p) (iChannel));
}

long ATFX_PHDRPY (int iChannel)
{
  ptr_ATFX_PHDRPY p;

  p = (ptr_ATFX_PHDRPY) *FaxFunc[ATFX_PHDRPY_ID].fptr;
  return ( (*p) (iChannel));
}

long ATFX_RESLN (int iChannel)
{
  ptr_ATFX_RESLN p;

  p = (ptr_ATFX_RESLN) *FaxFunc[ATFX_PHDRPY_ID].fptr;
  return ( (*p) (iChannel));
}

long ATFX_RTNPAGES (int iChannel)
{
  ptr_ATFX_RTNPAGES p;

  p = (ptr_ATFX_RTNPAGES) *FaxFunc[ATFX_RTNPAGES_ID].fptr;
  return ( (*p) (iChannel));
}

long ATFX_SCANLINES (int iChannel)
{
  ptr_ATFX_SCANLINES p;

  p = (ptr_ATFX_SCANLINES) *FaxFunc[ATFX_SCANLINES_ID].fptr;
  return ( (*p) (iChannel));
}

long ATFX_SPEED (int iChannel)
{
  ptr_ATFX_SPEED p;

  p = (ptr_ATFX_SPEED) *FaxFunc[ATFX_SPEED_ID].fptr;
  return ( (*p) (iChannel));
}

long ATFX_STATE (int iChannel)
{
  ptr_ATFX_STATE p;

  p = (ptr_ATFX_STATE) *FaxFunc[ATFX_STATE_ID].fptr;
  return ( (*p) (iChannel));
}

long ATFX_TERMMSK (int iChannel)
{
  ptr_ATFX_TERMMSK p;

  p = (ptr_ATFX_TERMMSK) *FaxFunc[ATFX_TERMMSK_ID].fptr;
  return ( (*p) (iChannel));
}

long ATFX_TFBADTAG (int iChannel)
{
  ptr_ATFX_TFBADTAG p;

  p = (ptr_ATFX_TFBADTAG) *FaxFunc[ATFX_TFBADTAG_ID].fptr;
  return ( (*p) (iChannel));
}

long ATFX_TFNOTAG (int iChannel)
{
  ptr_ATFX_TFNOTAG p;

  p = (ptr_ATFX_TFNOTAG) *FaxFunc[ATFX_TFNOTAG_ID].fptr;
  return ( (*p) (iChannel));
}

long ATFX_TFPGBASE (int iChannel)
{
  ptr_ATFX_TFPGBASE p;

  p = (ptr_ATFX_TFPGBASE) *FaxFunc[ATFX_TFPGBASE_ID].fptr;
  return ( (*p) (iChannel));
}

long ATFX_TRCOUNT (int iChannel)
{
  ptr_ATFX_TRCOUNT p;

  p = (ptr_ATFX_TRCOUNT) *FaxFunc[ATFX_TRCOUNT_ID].fptr;
  return ( (*p) (iChannel));
}

long ATFX_WIDTH (int iChannel)
{
  ptr_ATFX_WIDTH p;

  p = (ptr_ATFX_WIDTH) *FaxFunc[ATFX_WIDTH_ID].fptr;
  return ( (*p) (iChannel));
}

int fx_close (int iChannel)
{
  ptr_fx_close p;

  p = (ptr_fx_close) *FaxFunc[fx_close_ID].fptr;
  return ( (*p) (iChannel));
}

int fx_getDCS (int iChannel, DF_DCS *pDF_DCS)
{
  ptr_fx_getDCS p;

  p = (ptr_fx_getDCS) *FaxFunc[fx_getDCS_ID].fptr;
  return ( (*p) (iChannel, pDF_DCS));
}

int fx_getDIS (int iChannel, DF_DIS *pDF_DIS)
{
  ptr_fx_getDIS p;

  p = (ptr_fx_getDIS) *FaxFunc[fx_getDIS_ID].fptr;
  return ( (*p) (iChannel, pDF_DIS));
}

int fx_getNSF (int iChannel, unsigned short usFlags, char *pcBuf)
{
  ptr_fx_getNSF p;

  p = (ptr_fx_getNSF) *FaxFunc[fx_getNSF_ID].fptr;
  return ( (*p) (iChannel, usFlags, pcBuf));
}

int fx_getparm (int iChannel, unsigned long ulParm, void *pValue)
{
  ptr_fx_getparm p;

  p = (ptr_fx_getparm) *FaxFunc[fx_getparm_ID].fptr;
  return ( (*p) (iChannel, ulParm, pValue));
}

int fx_getxmitslot (int iChannel, SC_TSINFO *pSC_TSINFO)
{
  ptr_fx_getxmitslot p;

  p = (ptr_fx_getxmitslot) *FaxFunc[fx_getxmitslot_ID].fptr;
  return ( (*p) (iChannel, pSC_TSINFO));
}

int fx_initstat (int iChannel, int iState)
{
  ptr_fx_initstat p;

  p = (ptr_fx_initstat) *FaxFunc[fx_initstat_ID].fptr;
  return ( (*p) (iChannel, iState));
}

int fx_listen (int iChannel, SC_TSINFO *pSC_TSINFO)
{
  ptr_fx_listen p;

  p = (ptr_fx_listen) *FaxFunc[fx_listen_ID].fptr;
  return ( (*p) (iChannel, pSC_TSINFO));
}

int fx_open (const char *pcName, int iMode)
{
  ptr_fx_open p;

  p = (ptr_fx_open) *FaxFunc[fx_open_ID].fptr;
  return ( (*p) (pcName, iMode));
}

int fx_rcvfax (int iChannel, const char *pcName, unsigned long ulFlags)
{
  ptr_fx_rcvfax p;

  p = (ptr_fx_rcvfax) *FaxFunc[fx_rcvfax_ID].fptr;
  return ( (*p) (iChannel, pcName, ulFlags));
}

int fx_rcvfax2 (int iChannel, int iFileDescriptor, unsigned long ulFlag)
{
  ptr_fx_rcvfax2 p;

  p = (ptr_fx_rcvfax2) *FaxFunc[fx_rcvfax2_ID].fptr;
  return ( (*p) (iChannel, iFileDescriptor, ulFlag));
}

int fx_route (int iBoard, int iChannel, int iTimeSlot)
{
  ptr_fx_route p;

  p = (ptr_fx_route) *FaxFunc[fx_route_ID].fptr;
  return ((*p) (iBoard, iChannel, iTimeSlot));
}

int fx_sendfax (int iChannel, DF_IOTT *pDF_IOTT, unsigned long ulFlag)
{
  ptr_fx_sendfax p;

  p = (ptr_fx_sendfax) *FaxFunc[fx_sendfax_ID].fptr;
  return ((*p) (iChannel, pDF_IOTT, ulFlag));
}

void fx_setiott (DF_IOTT *pDF_IOTT, int iFileHandle, unsigned short usDataType, unsigned short usContValue)
{
  ptr_fx_setiott p;

  p = (ptr_fx_setiott) *FaxFunc[fx_setiott_ID].fptr;
  ((*p) (pDF_IOTT, iFileHandle, usDataType, usContValue));
  return;
}

int fx_setparm (int iChannel, unsigned long ulParm, const void *pValue)
{
  ptr_fx_setparm p;

  p = (ptr_fx_setparm) *FaxFunc[fx_setparm_ID].fptr;
  return ((*p) (iChannel, ulParm, pValue));
}

void fx_setuio (DF_UIO df_uio)
{
  ptr_fx_setuio p;

  p = (ptr_fx_setuio) *FaxFunc[fx_setuio_ID].fptr;
  ((*p) (df_uio));
  return;
}

int fx_stopch (int iChannel, int usMode)
{
  ptr_fx_stopch p;

  p = (ptr_fx_stopch) *FaxFunc[fx_stopch_ID].fptr;
  return ((*p) (iChannel, usMode));
}

int fx_unlisten (int iChannel)
{
  ptr_fx_unlisten p;

  p = (ptr_fx_unlisten) *FaxFunc[fx_unlisten_ID].fptr;
  return ((*p) (iChannel));
}

int fx_loadfont (int chdev, int fontfh, unsigned long mode)
{
  ptr_fx_loadfont p;

  p = (ptr_fx_loadfont) *FaxFunc[fx_loadfont_ID].fptr;
  return ((*p) (chdev, fontfh, mode));
}

int fx_rtvContinue (int chdev, int flag)
{
  ptr_fx_rtvContinue p;

  p = (ptr_fx_rtvContinue) *FaxFunc[fx_rtvContinue_ID].fptr;
  return ((*p) (chdev, flag));
}

int fx_originate (int chdev, int mode)
{
	ptr_fx_originate p;
	
	p = (ptr_fx_originate) *FaxFunc[fx_originate_ID].fptr;
	return ((*p) (chdev, mode));
}

