/***************************************************************************
 *		Copyright (c) 1996 Dialogic Corp.
 *		All Rights Reserved.
 *
 *		THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF Dialogic Corp.
 *		The copyright notice above does not evidence any actual or
 *		intended publication of such source code.
 ***************************************************************************/

/****************************************************************************
 * FILE:         qcomplib.h
 * ORG:          Dialogic Corporation
 * ORIG-DATE:    28-Aug-96 
 * LAST-MOD:     22-Jul-97 
 * PURPOSE:
 *    This file contains the defines stmts for component specific information 
 *
 * HISTORY:                   
 * Date              Who         Description
 ***************************************************************************/
					     
#ifndef __QCOMP_H__		   
#define __QCOMP_H__	 	    

// Define flag if compiling MNTI host library.
// Undefine flag if compiling applications.
#ifndef __MNTILIB_
#define MNTAPI __declspec(dllimport)
#else
#define MNTAPI __declspec(dllexport)
#endif

// Standard Node Identifiers 
#define QCOMP_N_SELF          qSelf.node

// standard Board Identifiers 
#define QCOMP_B_SELF          qSelf.board
#define QCOMP_B_HOST
#define QCOMP_B_NIL

//Standard Processors Identifiers 
#define QCOMP_P_HOST          0
#define QCOMP_P_CP            1
#define QCOMP_P_SP            
#define QCOMP_P_SELF          qSelf.processor
#define QCOMP_P_NIL           255

// Standard Component Identifiers 
#define QCOMP_C_SYS_SERVICE   0
#define QCOMP_C_TASK          1
#define QCOMP_C_STREAM			2
#define QCOMP_C_INVALID       
#define QCOMP_C_NIL           255

// Standard Instance Identifiers 
#define QCOMP_I_COMPONENT     0
#define QCOMP_I_HMSGDRIVER    1
#define QCOMP_I_HSTREAMDRV    2
#define QCOMP_I_IPCDRIVER     3
#define QCOMP_I_CONFIGMGR     4          
#define QCOMP_I_RESOURCEMGR   5
#define QCOMP_I_SMP           6
#define QCOMP_I_BSTREAM_TSK   7
#define QCOMP_I_CLUSTERMGR    8
#define QCOMP_I_SRAM          9
#define QCOMP_I_IDLE_TSK      10
#define QCOMP_I_FTIMER        11
#define QCOMP_I_QAGENT        12
#define QCOMP_I_NIL           255

// Component state 
#define QCOMPSTATE_UNREGISTERED  1
#define QCOMPSTATE_REGISTERED    2
#define QCOMPSTATE_ALLOCATED     3
#define QCOMPSTATE_FREE          4
#define QCOMPSTATE_RESERVED      5
#define QCOMPSTATE_RESTRICTED    6
#define QCOMPSTATE_FREEPENDING   7
#define QCOMPSTATE_ALLOCATING    8
#define QCOMPSTATE_FREEING       9

#define CLUSTERMGR	 { 0, \
                        0, \
                        0, \
                        QCOMP_P_CP, \
                        QCOMP_C_SYS_SERVICE, \
                        QCOMP_I_CLUSTERMGR, \
                        0}

// QCompDesc value for kernel services
#define RESOURCEMGR   { 0, \
                        0, \
                        0, \
                        QCOMP_P_CP, \
                        QCOMP_C_SYS_SERVICE, \
                        QCOMP_I_RESOURCEMGR, \
                        0}

typedef 	struct{
	USHORT   node;
	UCHAR    board;
   UCHAR    pad1;
	UCHAR	   processor;
	UCHAR	   component;
	UCHAR	   instance;
   UCHAR    pad2;
}QCompDesc, *PQCompDesc;

#define printDesc(a) { \
	qPrintf(("***%x:%x:%x:%x:%x***\n", \
				a.node, \
				a.board, \
				a.processor, \
				a.component, \
				a.instance)); \
}

typedef enum{QCOMP_PRIMARY=(1<<0),
             QCOMP_NOTIFY=(1<<1),
             QCOMP_ALLOACK=(1<<2),
             QCOMP_SHAREABLE=(1<<3),
             QCOMP_CLIENTEXIT=(1<<4),
             QCOMP_SUBEXIT=(1<<5)  
}QCompOptions;

typedef struct {
    ULONG  key;
    LONG   value;
}QCompAttr, *PQCompAttr;

//
//  Standard values for the key Field 
//
typedef enum {QATTR_UNIQUE,QATTR_SHARED}QCompAttrType;
   
//
// Special Values for the key field
//
#define QATTRKEY_OPERATORMASK 0xFFFFFFFC
#define QATTRKEY_OPERATOR     0xFFFFFFFC /* All values above 252 are operators */

#define QATTR_NULL 0xFFFFFFFF   
#define QATTR_AND  0xFFFFFFFE  
#define QATTR_OR   0xFFFFFFFD 
#define QATTR_NOT  0xFFFFFFFC 

//
//  Special Values for the value field
//
#define QATTR_ANY  0xFFFFFFFF


#ifdef __cplusplus
extern "C" {
#endif

// Functional Prototypes
MNTAPI BOOL WINAPI mntCompAllocate(HANDLE hDevice, 
                            QTrans nTransID,
                            PQCompDesc lpInstance,
	                         PQCompAttr lpAttr, 
                            QCompDesc ClusterDesc,
                            USHORT nTimeout, LPMMB lpMMB, LPOVERLAPPED lpOverlapped);
MNTAPI BOOL WINAPI mntCompFind(HANDLE hDevice, 
                        QTrans nTransID,
                        PQCompDesc lpInstance,
	                     PQCompAttr lpAttr, 
                        USHORT nTimeout, LPMMB lpMMB, LPOVERLAPPED lpOverlapped);
MNTAPI BOOL WINAPI mntCompFree(HANDLE hDevice, 
                        QTrans nTransID,
                        QCompDesc theInstance,
	                     USHORT nTimeout, LPMMB lpMMB, LPOVERLAPPED lpOverlapped);
MNTAPI BOOL WINAPI mntCompUse(HANDLE hDevice,
                       QTrans nTransID,
                       ULONG nCount,
                       PQCompDesc lpCompList,
                       PULONG lpPayload,
                       USHORT nTimeout, LPMMB lpMMB, LPOVERLAPPED	lpOverlapped);
MNTAPI BOOL WINAPI mntCompUnuse(HANDLE hDevice,
                         QTrans nTransID,
                         ULONG nCount,
                         PQCompDesc lpCompList,
                         USHORT nTimeout, LPMMB lpMMB, LPOVERLAPPED	lpOverlapped);
MNTAPI BOOL WINAPI mntCompFindAll(HANDLE hDevice,
                           QTrans nTransID,
                           QCompDesc startMask,
                           QCompDesc endMask,
	                        PQCompAttr lpAttr, 
                           USHORT nTimeout, LPMMB lpMMB, LPOVERLAPPED	lpOverlapped);
MNTAPI BOOL WINAPI mntNotifyRegister(HANDLE hDevice,
                              QTrans nTransID,
                              QCompDesc compDesc,
                              USHORT nTimeout, LPMMB lpMMB, LPOVERLAPPED	lpOverlapped);
MNTAPI BOOL WINAPI mntNotifyUnregister(HANDLE hDevice,
                                QTrans nTransID,
                                QCompDesc compDesc,
                                USHORT nTimeout, LPMMB lpMMB, LPOVERLAPPED	lpOverlapped);
MNTAPI BOOL WINAPI mntSetExitNotify(HANDLE hDevice, ULONG board, BOOL enable );

MNTAPI BOOL WINAPI mntCompAttrInfo(HANDLE hDevice,
                                   QTrans nTransID,
                                   PQCompDesc lpCompDesc,
                                   ULONG attrOffset,
                                   PULONG lpCount,
	                                PQCompAttr lpAttr, 
                                   USHORT nTimeout, LPMMB lpMMB, LPOVERLAPPED	lpOverlapped);

MNTAPI BOOL getTLSmmb(IN LPMMB *cMMB, IN ULONG CommandSize, IN ULONG ReplyMaxSize);

#ifdef __cplusplus
}
#endif

#endif	// __QCOMPLIB_H__
