/***********************************************************************
 *        FILE: scxlib.c
 * DESCRIPTION: Cross compatibility Library file for DIALOGIC Windows NT
 *
 *   Copyright (c) 1991-1996 Dialogic Corp. All Rights Reserved
 *
 *   THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF Dialogic Corp.
 *   The copyright notice above does not evidence any actual or
 *   intended publication of such source code.
 **********************************************************************/

#ifndef __CROSS_COMPAT_LIB__
#define __CROSS_COMPAT_LIB__
#endif

#include <windows.h>
#include "srllib.h"
#include "dtilib.h"
#include "scxlib.h"

//
// function prototypes
//
static int  scx_int_notimp(int ddd);
static char *scx_null_notimp(int ddd);

//
// typedefs and defines
//
typedef void (*func_ptr)(void);
struct func_table {
	char *fname;
	func_ptr fptr;
};

#define scx_close_ID              0
#define scx_clrevtmsk_ID          1
#define scx_getbrdparm_ID         2
#define scx_getctinfo_ID          3
#define scx_getevtmsk_ID          4
#define scx_getver_ID             5
#define scx_gtbdattr_ID           6
#define scx_open_ID               7
#define scx_setbrdparm_ID         8
#define scx_setevtmsk_ID          9
#define scx_tstcom_ID             10
#define scx_tstdat_ID             11

typedef int (*ptr_scx_close)(short int devh);
typedef int (*ptr_scx_clrevtmsk)(short int devh, unsigned short int mask);
typedef int (*ptr_scx_getbrdparm)(short int devh, unsigned char param, void *valuep);
typedef int (*ptr_scx_getctinfo)(short int devh, CT_DEVINFO *devinfop);
typedef int (*ptr_scx_getevtmsk)(short int devh, unsigned short int *bitmskp);
typedef int (*ptr_scx_getver)(short int devh);
typedef int (*ptr_scx_gtbdattr)(short int devh, tSCX_BRDST *datbufp);
typedef int (*ptr_scx_open)(char *name, short int oflags);
typedef int (*ptr_scx_setbrdparm)(short int devh, unsigned char param, void *parmvalp);
typedef int (*ptr_scx_setevtmsk)(short int devh, unsigned short int mask);
typedef int (*ptr_scx_tstcom)(short int devh, short int tmo);
typedef int (*ptr_scx_tstdat)(short int devh, short int tmo);

static struct func_table ScxFunc[] = {
{ "scx_close",                   (func_ptr)scx_int_notimp },          
{ "scx_clrevtmsk",               (func_ptr)scx_int_notimp },          
{ "scx_getbrdparm",              (func_ptr)scx_int_notimp }, 
{ "scx_getctinfo",               (func_ptr)scx_int_notimp },   
{ "scx_getevtmsk",               (func_ptr)scx_int_notimp }, 
{ "scx_getver",                  (func_ptr)scx_int_notimp },   
{ "scx_gtbdattr",                (func_ptr)scx_int_notimp },   
{ "scx_open",                    (func_ptr)scx_int_notimp },      
{ "scx_setbrdparm",              (func_ptr)scx_int_notimp },
{ "scx_setevtmsk",               (func_ptr)scx_int_notimp }, 
{ "scx_tstcom",                  (func_ptr)scx_int_notimp },      
{ "scx_tstdat",                  (func_ptr)scx_int_notimp },      
};

int scx_libinit(HINSTANCE LibHandle)
{
	struct func_table *scxp = ScxFunc;
	func_ptr ProcAddress;
	DWORD NumberofFunctions;

	NumberofFunctions = sizeof(ScxFunc)/sizeof(ScxFunc[0]);

	//
	// The library has been loaded previously. Set up each entry to a default
	// not implemented function and then overwrite the true address if
	// the function is present in the DLL.
	//
	while (NumberofFunctions --) {
		ProcAddress = (func_ptr)GetProcAddress(LibHandle, scxp->fname);
		if (ProcAddress) {
			scxp->fptr = ProcAddress;
		}
			    scxp++;
	}

	return 0;
}

int scx_close(short int devh)      
{
	ptr_scx_close p;

	p = (ptr_scx_close)*ScxFunc[scx_close_ID].fptr;
	return (*p)(devh);
}

int scx_clrevtmsk(short int devh, unsigned short int mask)
{
	ptr_scx_clrevtmsk p;

	p = (ptr_scx_clrevtmsk)*ScxFunc[scx_clrevtmsk_ID].fptr;
	return (*p)(devh,mask);
}

int scx_getbrdparm(short int devh, unsigned char param, void *valuep)
{
	ptr_scx_getbrdparm p;

	p = (ptr_scx_getbrdparm)*ScxFunc[scx_getbrdparm_ID].fptr;
	return (*p)(devh,param,valuep);
}

int scx_getevtmsk(short int devh, unsigned short *bitmaskp)
{
	ptr_scx_getevtmsk p;

	p = (ptr_scx_getevtmsk)*ScxFunc[scx_getevtmsk_ID].fptr;
	return (*p)(devh,bitmaskp);
}

int scx_getctinfo(short int devh, CT_DEVINFO *devinfop)
{
	ptr_scx_getctinfo p;

	p = (ptr_scx_getctinfo)*ScxFunc[scx_getctinfo_ID].fptr;
	return (*p)(devh, devinfop);
}

int scx_getver(short int devh)      
{
	ptr_scx_getver p;

	p = (ptr_scx_getver)*ScxFunc[scx_getver_ID].fptr;
	return (*p)(devh);
}

int scx_gtbdattr(short int devh, tSCX_BRDST *datbufp)
{
	ptr_scx_gtbdattr p;

	p = (ptr_scx_gtbdattr)*ScxFunc[scx_gtbdattr_ID].fptr;
	return (*p)(devh,datbufp);
}

int scx_open(char *name, short int oflags)
{
	ptr_scx_open p;

	p = (ptr_scx_open)*ScxFunc[scx_open_ID].fptr;
	return (*p)(name, oflags);
}

int scx_setbrdparm(short int devh, unsigned char param, void *valuep)
{
	ptr_scx_setbrdparm p;

	p = (ptr_scx_setbrdparm)*ScxFunc[scx_setbrdparm_ID].fptr;
	return (*p)(devh,param,valuep);
}

int scx_setevtmsk(short int devh, unsigned short int mask)
{
	ptr_scx_setevtmsk p;

	p = (ptr_scx_setevtmsk)*ScxFunc[scx_setevtmsk_ID].fptr;
	return (*p)(devh, mask);
}

int scx_tstcom(short int devh, short int tmo)
{
	ptr_scx_tstcom p;

	p = (ptr_scx_tstcom)*ScxFunc[scx_tstcom_ID].fptr;
	return (*p)(devh, tmo);
}

int scx_tstdat(short int devh, short int tmo)
{
	ptr_scx_tstdat p;

	p = (ptr_scx_tstdat)*ScxFunc[scx_tstdat_ID].fptr;
	return (*p)(devh, tmo);
}

scx_int_notimp(int devh)
{
	dt_set_lasterr(devh, EDT_INVMSG);
	return -1;
}

char *scx_null_notimp(int devh)
{
	dt_set_lasterr(devh,EDT_INVMSG);
	return (char *)NULL;
}

