/* -------------------------------------------------------------------------
   ------------------------------------------------------------------------- */
#include "brd_dialogic_code.h"
#include "brd_dialogic_d42_code.h"

/* -------------------------------------------------------------------------
   Name:        BrdDialogicD42::BrdDialogicD42                 (Constructor)

   Purpose:     Constructor
   ------------------------------------------------------------------------- */
BrdDialogicD42::BrdDialogicD42(PIVRINFO pIVRInfo) : BrdDialogic(pIVRInfo)
{
    // Nothing required here
}
/*BrdDialogicD42::BrdDialogicD42*/

/* -------------------------------------------------------------------------
   Name:        BrdDialogicD42::~BrdDialogicD42                 (Destructor)

   Purpose:     Destructor
   ------------------------------------------------------------------------- */
BrdDialogicD42::~BrdDialogicD42(void)
{
    // Nothing required here
}
/*BrdDialogicD42::~BrdDialogicD42*/

// ****************************************************************************
// ************************** LINE FUNCTIONS **********************************
// ****************************************************************************

/* -------------------------------------------------------------------------
   Name:        BrdDialogicD42::WaitForCall                         (public)

   Purpose:     Waits for the phone to ring.

   Modified:    Mar.01/2002 - TF Created (from BrdDialogic base class)
   				 Mar.25/2002 - TF Added retrieval of handset display

   Notes:       - Variables to receive DNIS and ANI are defined in
                	'StepWaitForCall::Execute' as follows:
							char  calledDigits[MAX_DIGIT_BUFFER_SIZE];
							char  callingDigits[MAX_DIGIT_BUFFER_SIZE];

   Notes:       - Variable to receive handset display is defined in
                	'StepWaitForCall::Execute' as follows:
                  	char  handsetDisplay[MAX_DIGIT_BUFFER_SIZE];

   ------------------------------------------------------------------------- */
IV_STATUS BrdDialogicD42::WaitForCall(long timeout, int numrings, bool pickup, char* calledDigits,
      char* callingDigits, char* handsetDisplay)
{
	int  rc;
	char *pOffset;
	char szBuf[49];
	char szCalling[MAX_DIGIT_BUFFER_SIZE];
	char szCalled[MAX_DIGIT_BUFFER_SIZE];
   int  numchars;

   IV_STATUS status;

	if(lineOffHook)
   {
      ivLog->Print("      ERROR: Port %d is already in use\n", pInfo->iPort);
   	return(IVR_FATAL);
   }

   if(calledDigits != NULL)
   {
      calledDigits[0] = '\0';
   }

   if(callingDigits != NULL)
   {
      callingDigits[0] = '\0';
   }

   if(handsetDisplay != NULL)
   {
      handsetDisplay[0] = '\0';
   }

   if((status = RequestLine(pInfo->iPort)) != IVR_SUCCESS)
   {
      return(status);
   }

   // @@@ WAS:
   // memset (szBuf, 0, MAX_DIGIT_BUFFER_SIZE);
   memset (szBuf, 0, 49);
   memset (szCalling, 0, MAX_DIGIT_BUFFER_SIZE);
   memset (szCalled, 0, MAX_DIGIT_BUFFER_SIZE);

   if(numrings == 0)
   {
      numrings = 1;
   }

	// Original method: SYNCHRONOUS. At least it works! - TF Dec.1/01
   // -----------------------------
	// @@@ WAS:
	//  // Wait for the phone to ring
	//   if (dx_wtring (portdevh, numrings, DX_OFFHOOK, timeout) == -1) {
	// Wait for the phone to ring, but leave on-hook until we get Caller ID - TF Feb.21/02
   if(dx_wtring (lines[curLine].chdev, numrings, DX_OFFHOOK, timeout) == -1)
   {
      ReleaseLine();
		ivLog->Print("            Wait ring failed for device 0x%x\n", lines[curLine].chdev);
      if (ATDV_LASTERR (lines[curLine].chdev) == EDX_TIMEOUT)
		{
         return IVR_TIMEOUT;
      }
      PrintErrorMsg ("ERROR: dx_wtring failed.", lines[curLine].chdev);
      return IVR_FAIL;
   }
	ivLog->Print ("            Incoming call...\n");

	ivLog->Print ("            d42_gtcallid, lines[curLine].chdev = %d, szBuf = 0x%x\n", lines[curLine].chdev, szBuf);
	rc = d42_gtcallid(lines[curLine].chdev, szBuf);
	if (rc == -1)
	{
		ivLog->Print ("            WARNING: Called/calling numbers could not be determined.\n");
      PrintErrorMsg ("\td42_gtcallid() failed.", lines[curLine].chdev);
		// szCalling[0] = '\0';
		// szCalled[0] = '\0';
	} else {
   	// Display the string we got - @@@ DEBUG
		ivLog->Print ("            Buffer = '%s', length = %d.\n", szBuf, strlen(szBuf));
      if (szBuf[0] == '_')
   		pOffset = strstr(szBuf+1, "_");
      else
   		pOffset = strstr(szBuf, "_");
		ivLog->Print ("            pOffset = 0x%x.\n", pOffset);
      if (pOffset != NULL)
      {
   		*pOffset = '\0';
      }

		// Get calling ID...
		strncpy (szCalling, szBuf, MAX_DIGIT_BUFFER_SIZE-1);
		szCalling[MAX_DIGIT_BUFFER_SIZE-1] = '\0';
      ivLog->Print ("            Calling number (network) ...: '%s'.\n", szCalling);   // @@@ DEBUG

		// Get called ID...
      if (pOffset != NULL)
      {
         strncpy (szCalled,  pOffset+1, MAX_DIGIT_BUFFER_SIZE-1);
         szCalled[MAX_DIGIT_BUFFER_SIZE-1] = '\0';
         ivLog->Print ("            Called number (network) ....: '%s'.\n", szCalled);    // @@@ DEBUG
      }
	}

	numchars = strlen(szCalling);
	if (numchars >= MAX_DIGIT_BUFFER_SIZE)
		numchars = MAX_DIGIT_BUFFER_SIZE - 1;
	strncpy(callingDigits, szCalling, numchars);
	ivLog->Print ("            Calling number (TeleFlow) ..: '%s'.\n", callingDigits);   // @@@

	numchars = strlen(szCalled);
	if (numchars >= MAX_DIGIT_BUFFER_SIZE)
		numchars = MAX_DIGIT_BUFFER_SIZE - 1;
	strncpy(calledDigits, szCalled, numchars);
	ivLog->Print ("            Called number (TeleFlow) ...: '%s'.\n", calledDigits);   // @@@

	// Section added to get D42 handset display - TF Mar.25/02
	ivLog->Print ("            d42_display, lines[curLine].chdev = %d, szBuf = 0x%x\n", lines[curLine].chdev, szBuf);
	rc = d42_display(lines[curLine].chdev, szBuf);
	if (rc == -1)
	{
		ivLog->Print ("            WARNING: Display information could not be retrieved.\n");
      PrintErrorMsg ("\td42_display failed.", lines[curLine].chdev);
	} else {
   	// Display the string we got - @@@ DEBUG
		ivLog->Print ("            Buffer = '%s', length = %d.\n", szBuf, strlen(szBuf));
   }
	// End new section added - TF Mar.25/02

	// Section added to get D42 handset display - TF Mar.26/02
	ivLog->Print ("            d42_display, lines[curLine].chdev = %d, szBuf = 0x%x\n", lines[curLine].chdev, szBuf);
	rc = d42_display(lines[curLine].chdev, szBuf);
	if (rc == -1)
	{
		ivLog->Print ("            WARNING: Display information could not be retrieved.\n");
      PrintErrorMsg ("\td42_display failed.", lines[curLine].chdev);
	} else {
   	// Display the string we got - @@@ DEBUG
		ivLog->Print ("            Buffer = '%s', length = %d.\n", szBuf, strlen(szBuf));
		// Copy to parameter pointer
		numchars = strlen(szBuf);
		if (numchars >= MAX_DIGIT_BUFFER_SIZE)
			numchars = MAX_DIGIT_BUFFER_SIZE - 2;
		strncpy(handsetDisplay, szBuf, numchars);
      handsetDisplay[MAX_DIGIT_BUFFER_SIZE - 1] = '\0';
		ivLog->Print ("            Display string (TeleFlow) ...: '%s'.\n", handsetDisplay);   // @@@
   }
	// End new section added - TF Mar.26/02

   return(pickup ? Pickup() : IVR_SUCCESS);
}
/*BrdDialogicD42::WaitForCall*/

/*EOF*/
