#include <ctype.h>   // isalpha, isdigit
#include <stdio.h>   // printf for debugging
#include <stdlib.h>  // atoi

#include "Spanish_code.h"

Spanish::Spanish(void)
{
   InitialiseWordTable();
}

void Spanish::InitialiseWordTable(void)
{
   wordTable[WB_CARDINAL + 0] = "cero";
   wordTable[WB_CARDINAL + 1] = "un";
   wordTable[WB_CARDINAL + 2] = "dos";
   wordTable[WB_CARDINAL + 3] = "tres";
   wordTable[WB_CARDINAL + 4] = "cuatro";
   wordTable[WB_CARDINAL + 5] = "cinco";
   wordTable[WB_CARDINAL + 6] = "seis";
   wordTable[WB_CARDINAL + 7] = "siete";
   wordTable[WB_CARDINAL + 8] = "ocho";
   wordTable[WB_CARDINAL + 9] = "nueve";
   wordTable[WB_CARDINAL + 10] = "diez";
   wordTable[WB_CARDINAL + 11] = "once";
   wordTable[WB_CARDINAL + 12] = "doce";
   wordTable[WB_CARDINAL + 13] = "trece";
   wordTable[WB_CARDINAL + 14] = "catorce";
   wordTable[WB_CARDINAL + 15] = "quince";
   wordTable[WB_CARDINAL + 16] = "diecisis";
   wordTable[WB_CARDINAL + 17] = "diecisiete";
   wordTable[WB_CARDINAL + 18] = "dieciocho";
   wordTable[WB_CARDINAL + 19] = "diecinueve";
   wordTable[WB_CARDINAL + 20] = "veinte";
   wordTable[WB_CARDINAL + 21] = "veinti";
   wordTable[WB_CARDINAL + 30] = "treinta";
   wordTable[WB_CARDINAL + 31] = "treinta y";
   wordTable[WB_CARDINAL + 40] = "cuarenta";
   wordTable[WB_CARDINAL + 41] = "cuarenta y";
   wordTable[WB_CARDINAL + 50] = "cincuenta";
   wordTable[WB_CARDINAL + 51] = "cincuenta y";
   wordTable[WB_CARDINAL + 60] = "sesenta";
   wordTable[WB_CARDINAL + 61] = "sesenta y";
   wordTable[WB_CARDINAL + 70] = "setenta";
   wordTable[WB_CARDINAL + 71] = "setenta y";
   wordTable[WB_CARDINAL + 80] = "ochenta";
   wordTable[WB_CARDINAL + 81] = "ochenta y";
   wordTable[WB_CARDINAL + 90] = "noventa";
   wordTable[WB_CARDINAL + 91] = "noventa y";
   wordTable[WB_CARDINAL + 101] = "ciento";
   wordTable[WB_CARDINAL + 102] = "doscientos";
   wordTable[WB_CARDINAL + 103] = "trescientos";
   wordTable[WB_CARDINAL + 104] = "cuatrocientos";
   wordTable[WB_CARDINAL + 105] = "quinientos";
   wordTable[WB_CARDINAL + 106] = "seiscientos";
   wordTable[WB_CARDINAL + 107] = "setecientos";
   wordTable[WB_CARDINAL + 108] = "ochocientos";
   wordTable[WB_CARDINAL + 109] = "novecientos";
   wordTable[WB_MONTH + 1] = "enero";
   wordTable[WB_MONTH + 2] = "febrero";
   wordTable[WB_MONTH + 3] = "marzo";
   wordTable[WB_MONTH + 4] = "abril";
   wordTable[WB_MONTH + 5] = "mayo";
   wordTable[WB_MONTH + 6] = "junio";
   wordTable[WB_MONTH + 7] = "julio";
   wordTable[WB_MONTH + 8] = "agosto";
   wordTable[WB_MONTH + 9] = "septiembre";
   wordTable[WB_MONTH + 10] = "octubre";
   wordTable[WB_MONTH + 11] = "noviembre";
   wordTable[WB_MONTH + 12] = "diciembre";
   wordTable[WB_ASCII + ' '] = "espacio en blanco";
   wordTable[WB_ASCII + '!'] = "signo de admiracin";
   wordTable[WB_ASCII + '\"'] = "comillas";
   wordTable[WB_ASCII + '#'] = "signo de numero";
   wordTable[WB_ASCII + '$'] = "signo de dolares";
   wordTable[WB_ASCII + '%'] = "por ciento";
   wordTable[WB_ASCII + '&'] = "ampersand";
   wordTable[WB_ASCII + '\''] = "apstrofe";
   wordTable[WB_ASCII + '('] = "abre parntesis";
   wordTable[WB_ASCII + ')'] = "cierra parntesis";
   wordTable[WB_ASCII + '*'] = "asterisco";
   wordTable[WB_ASCII + '+'] = "signo mas";
   wordTable[WB_ASCII + ','] = "coma";
   wordTable[WB_ASCII + '-'] = "guin";
   wordTable[WB_ASCII + '.'] = "periodo";
   wordTable[WB_ASCII + '/'] = "diagonal";
   wordTable[WB_ASCII + ':'] = "dos puntos";
   wordTable[WB_ASCII + ';'] = "punto y coma";
   wordTable[WB_ASCII + '<'] = "menor que";
   wordTable[WB_ASCII + '='] = "igual";
   wordTable[WB_ASCII + '>'] = "mayor que";
   wordTable[WB_ASCII + '?'] = "cierra signo de interrogacin";
   wordTable[WB_ASCII + '@'] = "arroba";
   wordTable[WB_ASCII + '['] = "abre corchete";
   wordTable[WB_ASCII + '\\'] = "diagonal invertida";
   wordTable[WB_ASCII + ']'] = "cierra corchete";
   wordTable[WB_ASCII + '^'] = "circunflejo";
   wordTable[WB_ASCII + '_'] = "guin bajo";
   wordTable[WB_ASCII + '{'] = "abre llave";
   wordTable[WB_ASCII + '|'] = "pipe";
   wordTable[WB_ASCII + '}'] = "cierra llave";
   wordTable[WB_ASCII + '~'] = "tilde";
   wordTable[WB_ASCII + ''] = "abre signo de interrogacin";
   wordTable[WB_ASCII + ''] = "";
   wordTable[WB_ASCII + ''] = "";
}

WordList Spanish::AlphaNumericWordList(string text)
{
   WordList wordList;

   for(string::iterator character = text.begin(); character != text.end(); character++)
   {
      if(*character == '1')
      {
         wordList += "uno";
      }
      else
      {
         wordList += LanguageModule::AlphaNumericWordList(text.substr(character - text.begin(), 1));
      }
   }

   return(wordList);
}

// -----------------------------------------------------------------------
//   Function:    DateWordList
//
//   Purpose:     Parses string and builds a message list to speak the date.
//                "CheckDate" should be called before calling this function.
//
//   Notes:      Does not check the validity of date (i.e. 2004 02 31 will be spoken as is)
//   -----------------------------------------------------------------------
WordList Spanish::DateWordList(string date, LanguageModule::SAY_FORMAT format)
{
   string year = date.substr(0, 4);
   string month = date.substr(5, 2);
   string day = date.substr(8, 2);
   WordList wordList;

   switch(format)
   {
      // DD de month-name YYYY
      case SAY_DATE:
      {
         wordList = UnToUno(IntegerWordList(day));
         wordList += "de";
         wordList += wordTable[WB_MONTH + atoi(month.c_str())];
         wordList += UnToUno(IntegerWordList(year));
         break;
      }

      // DD de month-name
      case SAY_MONTHDAY:
      {
         wordList = UnToUno(IntegerWordList(day));
         wordList += "de";
         wordList += wordTable[WB_MONTH + atoi(month.c_str())];
         break;
      }

      // month-name YYYY
      case SAY_YEARMONTH:
      {
         wordList += wordTable[WB_MONTH + atoi(month.c_str())];
         wordList += UnToUno(IntegerWordList(year));
         break;
      }
   }

   return(wordList);
}

WordList Spanish::DigitsWordList(string digits)
{
   WordList wordList;

   for(string::iterator digit = digits.begin(); digit != digits.end(); digit++)
   {
      if(*digit == '1')
      {
         wordList += "uno";
      }
      else
      {
         wordList += LanguageModule::DigitsWordList(digits.substr(digit - digits.begin(), 1));
      }
   }

   return(wordList);
}

WordList Spanish::IntegerWordList(string integer, bool ordinal)
{
   WordList wordList = LanguageModule::IntegerWordList(integer, ordinal);
   string wordListString = wordList.c_str();

   if((wordListString.length() >= 6) && (wordListString.substr(0, 6) == "un,mil"))
   {
      wordList = wordListString.substr(3);
   }

   return(wordList);
}

WordList Spanish::MoneyWordList(string money)
{
   WordList wordList;
   int endZeroes = 0;
   string integer = Integer(money);
   string fraction = Fraction(money);

   if(!integer.empty())
   {
      wordList = IntegerWordList(integer);

      for(string::iterator index = integer.end() - 1;
         (index >= integer.begin()) && ((*index == '0') || (*index == ',')); index--)
      {
         endZeroes += *index == '0' ? 1 : 0;
      }

      if(endZeroes >= 6)
      {
         wordList += "de";
      }

      wordList += (atoi(integer.c_str()) == 1) || (atoi(integer.c_str()) == -1) ? "dolar" : "dolares";

      if(!fraction.empty())
      {
         wordList += "y de conjuncin";
      }
   }

   if(!fraction.empty())
   {
      wordList += IntegerWordList(fraction);
      wordList += (atoi(fraction.c_str()) == 1) || (atoi(fraction.c_str()) == -1) ? "centavo" : "centavos";
   }

   return(wordList);
}

WordList Spanish::NumberGroupWordList(string numberGroup, bool)
{
   WordList wordList;
   bool useTeen = false;
   string::iterator digit = numberGroup.begin();

   if(*digit == '+')
   {
      wordList += "positivo";
      digit++;
   }
   else if(*digit == '-')
   {
      wordList += "negativo";
      digit++;
   }

   for(; digit != numberGroup.end(); digit++)
   {
      switch(numberGroup.end() - digit)
      {
         // Hundreds column
         case 3:
         {
            if(numberGroup.find("100") != string::npos)
            {
               wordList += "cien";
               return(wordList);
            }

            if(*digit != 0)
            {
               wordList += wordTable[WB_CARDINAL + (*digit - '0') + 100];
            }

            break;
         }

         // Tens column
         case 2:
         {
            if(*digit != '0')
            {
               if(*digit == '1')
               {
                  useTeen = true;
               }
               else if(*(digit + 1) == '0')
               {
                  wordList += wordTable[WB_CARDINAL + (*digit - '0')*10];
               }
               else
               {
                  wordList += wordTable[WB_CARDINAL + (*digit - '0')*10 + 1];
               }
            }
            break;
         }

         // Ones column
         case 1:
         {
            if(useTeen)
            {
               wordList += wordTable[WB_CARDINAL + 10 + *digit - '0'];
            }
            else if(*digit != '0')
            {
               wordList += wordTable[WB_CARDINAL + *digit - '0'];
            }
            break;
         }
      }
   }

   return(wordList);
}

WordList Spanish::NumberGroupWordList(int index, bool plural)
{
   WordList wordList;

   switch(index)
   {
      case 1:
      {
         wordList = "mil";
         break;
      }

      case 2:
      {
         wordList = plural ? "millones" : "milln";
         break;
      }

      case 3:
      {
         wordList = "mil";
         wordList += "millones";
         break;
      }

      case 4:
      {
         wordList = plural ? "billones" : "billn";
         break;
      }
   }

   return(wordList);
}

WordList Spanish::RealNumberWordList(string number)
{
   WordList wordList = UnToUno(IntegerWordList(Integer(number)));

   if(!Fraction(number).empty())
   {
      string fraction = Fraction(number);

      wordList += "punto";

      for(string::iterator digit = fraction.begin(); digit != fraction.end(); digit++)
      {
         wordList += wordTable[WB_CARDINAL + *digit - '0'];
      }
   }

   return(wordList);
}

WordList Spanish::TimeWordList(string time, LanguageModule::SAY_FORMAT format)
{
   string hour = time.substr(0,2);
   string minute = time.substr(3,2);
   string second;
   int iHour = atoi(hour.c_str());
   WordList wordList;

   if(time.length() > 5)
   {
      second = time.substr(6,2);
   }

   if((format == SAY_12HOUR) && (iHour > 13))
   {
      wordList = wordTable[WB_CARDINAL + iHour - 12];
      wordList += "horas";
   }
   else if(iHour == 0)
   {
      wordList = format == SAY_12HOUR ? "doce" : "cero";
      wordList += "horas";
   }
   else if((iHour == 1) || ((format == SAY_12HOUR) && (iHour == 13)))
   {
      wordList = "una";
      wordList += "hora";
   }
   else
   {
      wordList = IntegerWordList(hour);
      wordList += "horas";
   }

   if(atoi(minute.c_str()) > 0)
   {
      wordList += IntegerWordList(minute);
      wordList += atoi(minute.c_str()) == 1 ? "minuto" : "minutos";
   }

   if(!second.empty())
   {
      wordList += "y de conjuncin";
      wordList += IntegerWordList(second);
      wordList += atoi(second.c_str()) == 1 ? "segundo" : "segundos";
   }

   if(format == SAY_12HOUR)
   {
      wordList += iHour >= 12 ? "pm" : "am";
   }

   return(wordList);
}

WordList Spanish::UnToUno(WordList wordList)
{
   string wordListString = wordList.c_str();

   if((wordListString == "un")
      || ((wordListString.length() > 3) && (wordListString.substr(wordListString.length() - 3) == ",un")))
   {
      wordListString += 'o';
      wordList = wordListString;
   }

   return(wordList);
}

LanguageModule* CreateLanguageModule(void)
{
   return(new Spanish());
}

void ReleaseLanguageModule(LanguageModule* lang)
{
   delete (Spanish*) lang;
}