/* -------------------------------------------------------------------------
   File:        conf_main.cpp

   TODO:        - Need to ensure that ( ... ) buttons work correctly.
                - If the user types a file name into an entry field, ensure
                  that it has an extension.
   ------------------------------------------------------------------------- */
#include <vcl.h>
#include <stdio.h>						// sprintf
#include <process.h>                // spawnlp
#include <math.h>
#include <string>
using std::string;
#pragma hdrstop

#include "..\escutil\help_fns.h"    // for link to Help file entry
#include "dl_advanced.h"				// FormAdvanced
#include "conf_about.h"					// AboutBox
#include "conf_help.h"					// FormHelpContents
#include "conf_main.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TFormConf *FormConf;
//---------------------------------------------------------------------------

__fastcall TFormConf::TFormConf(TComponent* Owner)
    : TForm(Owner)
{
   try
   {  // Assume that we will want our registry settings saved unless Cancel is pressed
   	bUpdateReg = TRUE;

		// Read in the current registry settings
      ReadRegistry();
   }
   catch (Exception &e)
   {
      Application->MessageBox (
         e.Message.c_str(),
         "Constructor",
         MB_OK | MB_ICONERROR);
   }
}
//---------------------------------------------------------------------------

void __fastcall TFormConf::btnLogDirClick(TObject *Sender)
{
   AnsiString  Title ("Open Application Log Directory");
   WideString  Root = "\\";
   AnsiString  Directory;

   if(SelectDirectory (Title, Root, Directory))
   {
   	efLogDir->Text  = Directory;
   }

	// Add an extra \ at the end of the directory path for the Server if it isn't there
	if(*(efLogDir->Text.AnsiLastChar()) != '\\')
	{
      efLogDir->Text = efLogDir->Text + "\\";
   }
}
//---------------------------------------------------------------------------

void __fastcall TFormConf::efLogLinesExit(TObject *Sender)
{
	// Check to make sure the value is valid
	if (ValidLogLines() && efLogLines->Text.ToInt() >= 2000)
   {
      float fMBRequired;
      string sWarningMsg;
      char   temp[20];

      // number of lines x bytes/line, converted to MB:
      fMBRequired = ((efLogLines->Text.ToInt() * 100) / pow(2,20));

      sWarningMsg = "Memory consumption will be 100 bytes / line / application.\n";
      sWarningMsg += "Your setting will require approximately ";
      sprintf(temp, "%.2f", fMBRequired);
      sWarningMsg += temp;
      sWarningMsg += " MB per application (for logging alone).";

      Application->MessageBox(sWarningMsg.c_str(), "Memory Usage Warning",
         MB_OK | MB_ICONWARNING);
   }
}
//---------------------------------------------------------------------------

bool __fastcall TFormConf::ValidLogLines(void)
{
	try
   {
      if(efLogLines->Text.ToInt() < 50)
      {
         Application->MessageBox("Value must be between 50 and 20000.",
            "Log Lines", MB_OK | MB_ICONERROR);

         // Smallest allowable value
         efLogLines->Text = "50";

			// Set cursor to efLogLines field
			FocusControl(efLogLines);
         return(false);
      }

      if(efLogLines->Text.ToInt() > 20000)
      {
         Application->MessageBox("Value must be between 50 and 20000.",
            "Log Lines", MB_OK | MB_ICONERROR);

			// Largest allowable value
         efLogLines->Text = "20000";

			// Set cursor to efLogLines field
			FocusControl(efLogLines);
         return(false);
      }
   }

   catch(Exception &e)
   {
      Application->MessageBox("Value must be between 50 and 20000.",
         "Log Lines", MB_OK | MB_ICONERROR);

		// Default value
      efLogLines->Text = "200";

		// Set cursor to efLogLines field
		FocusControl(efLogLines);
      return(false);
   }

   return(true);
}
//---------------------------------------------------------------------------

void __fastcall TFormConf::btnActiveTALClick(TObject *Sender)
{
   dlgOpen->FileName = "";
   dlgOpen->Title = "Open TeleFlow Application List File";
   dlgOpen->Filter = "TeleFlow Application List File (*.tal)|*.TAL|"
                     "All Files (*.*)|*.*";
   dlgOpen->Options.Clear();
   dlgOpen->Options << ofFileMustExist << ofHideReadOnly << ofNoChangeDir;

   if (dlgOpen->Execute()) {
      efActiveTAL->Text = dlgOpen->FileName;
   }
}
//---------------------------------------------------------------------------

void __fastcall TFormConf::mnuFileExitClick(TObject *Sender)
{
	Close();
}
//---------------------------------------------------------------------------

/* -------------------------------------------------------------------------
   Name:        TFormConf::FormCloseQuery

   Purpose:	Windows checks boolean CanCloseto see if close the window by calling
   			FormCloseQuery first.
   				Checks for changes.  If changes have been made, the user is
                prompted to save their changes before exiting the
                application.
   ------------------------------------------------------------------------- */
void __fastcall TFormConf::FormCloseQuery (TObject *Sender,  bool &CanClose)
{
   CanClose = 	ValidLogLines();

	// Update registry settings
	if(CanClose && bUpdateReg)
	{
   	WriteRegistry();
   }
}
//---------------------------------------------------------------------------

/* -------------------------------------------------------------------------
   Name:        TFormConf::ReadRegistry

   Purpose:     Reads values from the registry into the display fields on the form.

   Notes:       - When a new field is added to the form, it won't exist in
                  the registry.  Don't stop loading registry settings if one
                  value is missing, and use a default value.
   ------------------------------------------------------------------------- */
void __fastcall TFormConf::ReadRegistry(void)
{
   LONG  ret;
   HKEY  hkey;
   HKEY  hkey2;
   DWORD dwLen;
   DWORD dwType;
   DWORD dwValue;
   char  szKey[21];  // Capable of holding "THardware %d" for up to 32-bit int ii
   char* pszValue;

   try
   {
   	ret = RegOpenKeyEx(HKEY_LOCAL_MACHINE, TFCONFIG_REG_KEY, 0, KEY_READ, &hkey);
      ret &= RegOpenKeyEx(HKEY_LOCAL_MACHINE, TFSERVICE_REG_KEY, 0, KEY_READ, &hkey2);

      if(ret != ERROR_SUCCESS)
      {
      	Application->MessageBox("Failed to create registry key.", "Registry Save Failed", MB_ICONERROR | MB_OK);
      }
      else
      {
         ret = RegQueryValueEx(hkey, "NumInstalledHardwareTypes", 0, &dwType, NULL, &dwLen);

         if(ret == ERROR_SUCCESS)
         {
         	ret = RegQueryValueEx(hkey, "NumInstalledHardwareTypes", 0, &dwType, (LPBYTE) &dwValue, &dwLen);
         }
         if((ret == ERROR_SUCCESS) && ((int) dwValue > 0))
         {
         	// Get name of each installed hardware item
         	for(int ii = 0; ii < (int) dwValue; ii++)
            {
            	sprintf(szKey, "THardware %d", ii);

               // Get length of string
               ret = RegQueryValueEx(hkey, szKey, NULL, &dwType, NULL, &dwLen);

               if(ret == ERROR_SUCCESS)
               {
               	// Create copy of string
                  pszValue = new char[dwLen + 1];
                  ret = RegQueryValueEx(hkey, szKey, NULL, &dwType, pszValue, &dwLen);

                  if(ret == ERROR_SUCCESS)
                  {
                  	ComboBoxTHardware->Items->Add(pszValue);
                  }

                  delete[] pszValue;
               }
            }
         }
         // Default value if registry key doesn't exists or can't be read
         else
         {
         	ComboBoxTHardware->Items->Add("No hardware installed");
         	ComboBoxTHardware->ItemIndex = 0;
         }

         // Get length of THardware string in registry
         ret = RegQueryValueEx(hkey, "THardware", NULL, &dwType, NULL, &dwLen);
         if(ret == ERROR_SUCCESS)
         {
         	// Create copy of string
            pszValue = new char[dwLen + 1];
            ret = RegQueryValueEx (hkey, "THardware", NULL, &dwType, pszValue, &dwLen);

            // Copy string to THardware combo box
            if(ret == ERROR_SUCCESS)
            {
            	ComboBoxTHardware->ItemIndex = ComboBoxTHardware->Items->IndexOf(pszValue);
            }

            delete[] pszValue;
         }
         // Default value if registry key doesn't exists or can't be read
         else
         {
         	ComboBoxTHardware->ItemIndex = -1;
         }

         // Get length of AppListFileName string in registry
         ret = RegQueryValueEx(hkey, "AppListFileName", NULL, &dwType, NULL, &dwLen);
         if(ret == ERROR_SUCCESS)
         {
         	// Create copy of string
            pszValue = new char[dwLen + 1];
            ret = RegQueryValueEx (hkey, "AppListFileName", NULL, &dwType, pszValue, &dwLen);

            // Copy string to AppListFileName text box
            if(ret == ERROR_SUCCESS)
            {
            	efActiveTAL->Text = pszValue;
            }

            delete[] pszValue;
         }
         // Default value if registry key doesn't exists or can't be read
         else
         {
         	efActiveTAL->Text = "";
         }

         // Get length of LogDirectory string in registry
         ret = RegQueryValueEx(hkey, "LogDirectory", NULL, &dwType, NULL, &dwLen);
         if(ret == ERROR_SUCCESS)
         {
         	// Create copy of string
            pszValue = new char[dwLen + 1];
            ret = RegQueryValueEx (hkey, "LogDirectory", NULL, &dwType, pszValue, &dwLen);

            // Copy string to LogDirectory text box
            if(ret == ERROR_SUCCESS)
            {
            	efLogDir->Text = pszValue;
            }

            delete[] pszValue;
         }
         // Default value if registry key doesn't exists or can't be read
         else
         {
         	efLogDir->Text = "";
         }

         // Get length of LogLines string in registry
         ret = RegQueryValueEx(hkey, "LogLines", NULL, &dwType, NULL, &dwLen);
         if(ret == ERROR_SUCCESS)
         {
         	// Create copy of string
            pszValue = new char[dwLen + 1];
            ret = RegQueryValueEx (hkey, "LogLines", NULL, &dwType, (LPBYTE) &dwValue, &dwLen);

            // Copy string to LogLines text box
            if(ret == ERROR_SUCCESS)
            {
            	efLogLines->Text = dwValue;
            }

            delete[] pszValue;
         }
         // Default value if registry key doesn't exists or can't be read
         else
         {
         	efLogLines->Text = "200";
         }

         // Get length of MaxLogFileSize string in registry
         ret = RegQueryValueEx(hkey, "MaxLogFileSize", NULL, &dwType, NULL, &dwLen);
         if(ret == ERROR_SUCCESS)
         {
         	// Create copy of string
            pszValue = new char[dwLen + 1];
            ret = RegQueryValueEx (hkey, "MaxLogFileSize", NULL, &dwType, (LPBYTE) &dwValue, &dwLen);

            // Copy string to efMaxLogFileSize text box
            if(ret == ERROR_SUCCESS)
            {
					// Value of -1 means unlimited log file size
               if((int) dwValue == -1)
               {
            		efMaxLogFileSize->Text =  "";
						efMaxLogFileSize->Enabled = false;
						efMaxLogFileSize->Color = clInactiveCaptionText;
                  maxLogFileSize = 500;
               }
               else
               {
            		efMaxLogFileSize->Text = dwValue;
						efMaxLogFileSize->Enabled = true;
						efMaxLogFileSize->Color = clWindow;
                  maxLogFileSize = dwValue;
               }

               cbUnlimitedLogFileSize->Checked = ((int) dwValue == -1);
            }

            delete[] pszValue;
         }
         // Default value if registry key doesn't exists or can't be read
         else
         {
         	efMaxLogFileSize->Text = "500";
         }

         // Get LogSRAudio value in registry
         ret = RegQueryValueEx(hkey, "LogSRAudio", NULL, &dwType, (LPBYTE) &dwValue, &dwLen);
         if(ret == ERROR_SUCCESS)
         {
            cbLogSRAudio->Checked = dwValue != 0;
         }
         else
         {
            cbLogSRAudio->Checked = false;
         }

         // get the value of the service start type
         ret = RegQueryValueEx(hkey2, "Start", 0, &dwType, (LPBYTE) &dwValue, &dwLen);
         if(ret == ERROR_SUCCESS)
         {
            switch((int) dwValue)
            {
               case 1:
               {
                  cbStartServiceAutomatically->Checked = true;   // system.... should NOT get into this state
                                                                 // unless the registry is hacked
                                                                 // change value to AUTO on settings save
                  break;
               }
               case 2:
               {
                  cbStartServiceAutomatically->Checked = true;   // auto
                  break;
               }
               case 3:
               {
                  cbStartServiceAutomatically->Checked = false;   // manual
                  break;
               }
               case 4:
               {
                  // cbStartServiceAutomatically->Checked = false;   // disabled
                  cbStartServiceAutomatically->Enabled = false;
                  break;
               }
            } // end switch
         } // end if(ret == ERROR_SUCCESS)
      } // end else of if(ret != ERROR_SUCCESS)
   } // end try

   catch(Exception &e)
   {
      Application->MessageBox(e.Message.c_str(), "ReadRegistry", MB_OK | MB_ICONERROR);
   }
}
//---------------------------------------------------------------------------

/* -------------------------------------------------------------------------
   Name:        TFormConf::WriteRegistry

   Purpose:     Writes values from display fields into the registry.

   Notes:       - When a new field is added to the form, it won't exist in
                  the registry.  Don't stop loading registry settings if one
                  value is missing, and use a default value.
   ------------------------------------------------------------------------- */
void __fastcall TFormConf::WriteRegistry(void)
{
   LONG  ret;
   HKEY  hkey;
   HKEY  hkey2;
   DWORD dwDisposition;
   DWORD dwDisposition2;
   DWORD dwValue;
   DWORD dwLen;
   DWORD dwLen2;
	DWORD dwType;
   char* pszValue = NULL;
   // 2003-06-05 JSH added for the 'board services'
   string strServiceName;
   char* pszServiceName = NULL;
   char* pszCurrBoardService;
   DWORD dwLen3;
   DWORD dwLen4;

   string strAlertMessage;

   // get the service name for the particular hardware
   strServiceName = ComboBoxTHardware->Text.c_str();
   strServiceName += "_SERVICE";

   try
   {
   	ret = RegOpenKeyEx(HKEY_LOCAL_MACHINE, TFCONFIG_REG_KEY, 0, KEY_READ, &hkey);
      ret &= RegOpenKeyEx(HKEY_LOCAL_MACHINE, TFSERVICE_REG_KEY, 0, KEY_READ, &hkey2); // service 'start state' (auto, manual)

      if(ret != ERROR_SUCCESS)
      {
      	Application->MessageBox("Failed to create registry key.", "Registry Save Failed", MB_ICONERROR | MB_OK);
      }
      else
      {
      	// Get length of InstallDirectory string
         ret = RegQueryValueEx(hkey, "InstallDirectory", NULL, &dwType, NULL, &dwLen);

      	// Get length of THardware string in registry
         ret &= RegQueryValueEx(hkey, ComboBoxTHardware->Text.c_str(), NULL, &dwType, NULL, &dwLen2);

         // Get length of Services string in registry
         ret &= RegQueryValueEx(hkey, strServiceName.c_str(), NULL, &dwType, NULL, &dwLen3);

         // Get length of BOARDSERVICE string in registry
         ret &= RegQueryValueEx(hkey, "BOARDSERVICE", NULL, &dwType, NULL, &dwLen4);

         // Get length of Start DWORD in registry (services variable)
         // ret &= RegQueryValueEx(hkey2, "Start", NULL, &dwType, NULL, &dwLen4);

         if(ret == ERROR_SUCCESS)
         {
         // 2003-06-04 JSH note: *right*, this queries the two registry values "InstallDirectory" AND
         // the one that corresponds to currently selected hardware type (which returns the dll name)
         // and must combine the two in pszValue to form the registry entry for BOARDDLL
         	if(ComboBoxTHardware->ItemIndex != -1)
				{
            	// Create copy of directory   Note: lengths include null characters
            	pszValue = new char[dwLen + dwLen2 - 1];
               RegQueryValueEx(hkey, "InstallDirectory", NULL, &dwType, pszValue, &dwLen);

					// Add .DLL name
            	RegQueryValueEx (hkey, ComboBoxTHardware->Text.c_str(), NULL,
               	&dwType, pszValue + dwLen - 1, &dwLen2);

               // 2003-06-04 JSH
               // get the service name for the particular type of hardware
               pszServiceName = new char[dwLen3];
               RegQueryValueEx (hkey, strServiceName.c_str(), NULL,
               	&dwType, pszServiceName, &dwLen3);

	         }
				// No hardware selected
            else
            {
            	pszValue = new char[1];
               pszValue[0] = '\0';
               pszServiceName = new char[1];
               pszServiceName[0] = '\0';
            }

            pszCurrBoardService = new char[dwLen4];
            RegQueryValueEx (hkey, "BOARDSERVICE", NULL, &dwType, pszCurrBoardService, &dwLen4);

         }
         // Default value if registry key doesn't exists or can't be read
         else
         {
         	pszValue = NULL;
         }

         RegCloseKey (hkey);
      }

      ret = RegCreateKeyEx(
         HKEY_LOCAL_MACHINE,        // Handle of an open key.
         TFCONFIG_REG_KEY,          // Address of subkey name.
         0,                         // Reserved.
         0,                         // Address of class string.
         REG_OPTION_NON_VOLATILE,   // Special options flag.
         KEY_WRITE,                 // Desired security access.
         0,                         // Address of key security structure.
         &hkey,                     // Address of buffer for opened handle.
         &dwDisposition);           // Address of disposition value buffer.

      // for service start value...
      ret &= RegCreateKeyEx(
         HKEY_LOCAL_MACHINE,        // Handle of an open key.
         TFSERVICE_REG_KEY,          // Address of subkey name.
         0,                         // Reserved.
         0,                         // Address of class string.
         REG_OPTION_NON_VOLATILE,   // Special options flag.
         KEY_WRITE,                 // Desired security access.
         0,                         // Address of key security structure.
         &hkey2,                     // Address of buffer for opened handle.
         &dwDisposition2);           // Address of disposition value buffer.

      if(ret != ERROR_SUCCESS)
      {
      	Application->MessageBox("Failed to create registry key.", "Registry Save Failed", MB_ICONERROR | MB_OK);
      }
      else
      {
      	ret = RegSetValueEx(hkey, "BoardDLL", 0, REG_SZ, pszValue, strlen(pszValue) + 1);
       	delete[] pszValue;


         if(strcmp(pszServiceName, pszCurrBoardService) != 0)
         {
          // 2003-06-04 JSH add the write value for BOARDSERVICE
          ret &= RegSetValueEx(hkey, "BOARDSERVICE", 0, REG_SZ, pszServiceName, strlen(pszServiceName) + 1);
          // change the dependecy within the service's reg settings
          ret &= RegSetValueEx(hkey2, "DependOnService", 0, REG_SZ, pszServiceName, strlen(pszServiceName) + 1);
       	 delete[] pszServiceName;
          delete[] pszCurrBoardService;

          strAlertMessage = "Changing hardware types requires that you reboot your computer.";

          //Application->MessageBox("Changing hardware types requires that you reboot your computer.", "Restart Computer.", MB_ICONWARNING | MB_OK);

         }
         else
         {
          strAlertMessage = "Changes to TFConfig may require that you restart TFServer from the Services Control Manager.";
         }

         ret &= RegSetValueEx(hkey, "THardware", 0, REG_SZ,
         	ComboBoxTHardware->Text.c_str(), ComboBoxTHardware->Text.Length() + 1);

         ret &= RegSetValueEx(hkey, "AppListFileName", 0, REG_SZ,
         	efActiveTAL->Text.c_str(), efActiveTAL->Text.Length() + 1);

			// Add an extra space in case we need to add an end slash
        	pszValue = new char[efLogDir->Text.Length() + 2];
			strcpy(pszValue, efLogDir->Text.c_str());

			// Check for end slash and add it if it isn't there
         if(pszValue[strlen(pszValue) - 1] != '\\')
         {
         	strcat(pszValue, "\\");
         }
         ret &= RegSetValueEx(hkey, "LogDirectory", 0, REG_SZ, pszValue, strlen(pszValue) + 1);
         delete[] pszValue;

         dwValue = atoi(efLogLines->Text.c_str());
         ret &= RegSetValueEx(hkey, "LogLines", 0, REG_DWORD, (LPBYTE) &dwValue, sizeof(DWORD));

			// Value of -1 means unlimited log file size
         dwValue = cbUnlimitedLogFileSize->Checked ? -1 : atoi(efMaxLogFileSize->Text.c_str());
         ret &= RegSetValueEx(hkey, "MaxLogFileSize", 0, REG_DWORD, (LPBYTE) &dwValue, sizeof(DWORD));

      	dwValue = atoi(FormAdvanced->efMonitorPort->Text.c_str());
         ret &= RegSetValueEx(hkey, "MonitorPort", 0, REG_DWORD, (LPBYTE) &dwValue, sizeof(DWORD));

      	dwValue = atoi(FormAdvanced->efMonitorHTTPPort->Text.c_str());
         ret &= RegSetValueEx(hkey, "MonitorHTTPPort", 0, REG_DWORD, (LPBYTE) &dwValue, sizeof(DWORD));

         dwLen = FormAdvanced->efINIFile->Text.Length() + 1;
         ret &= RegSetValueEx(hkey, "INIFile", 0, REG_SZ,
         	FormAdvanced->efINIFile->Text.c_str(),
            FormAdvanced->efINIFile->Text.Length() + 1);

      	dwValue = cbLogSRAudio->Checked ? 1 : 0;
         ret &= RegSetValueEx(hkey, "LogSRAudio", 0, REG_DWORD, (LPBYTE) &dwValue, sizeof(DWORD));

         // service start type
         if(!cbStartServiceAutomatically->Enabled) // if NOT enabled, keep value to 4 (disabled)
         {
            dwValue = 4; // automatic
         }
         else
         {
            if(cbStartServiceAutomatically->Checked)
            {
               dwValue = 2; // automatic
            }
            else
            {
               dwValue = 3; // manual
            }
         }
         ret &= RegSetValueEx(hkey2, "Start", 0, REG_DWORD, (LPBYTE) &dwValue, sizeof(DWORD));

         if(ret != ERROR_SUCCESS)
         {
         	Application->MessageBox("Failed to write host entry.", "Registry Save Failed", MB_ICONERROR | MB_OK);
         }
         // 2003-06-05 JSH add msg for service restart
         else
         {
            if(cbStartServiceAutomatically->Checked)
            {
               Application->MessageBox(strAlertMessage.c_str(), "TFConfig Alert", MB_ICONWARNING | MB_OK);
            }
            else
            {
               Application->MessageBox("The TFServer Service is currently disabled.\nYou must start TFServer manually via the command line or re-enable TFServer from the Services Control Manager.", "TFConfig Alert", MB_ICONWARNING | MB_OK);
            }
         }
      }

      RegCloseKey (hkey);
      RegCloseKey (hkey2);
   }

   catch(Exception &e)
   {
   	Application->MessageBox(e.Message.c_str(), "ReadRegistry", MB_OK | MB_ICONERROR);
   }
}
//---------------------------------------------------------------------------

void __fastcall TFormConf::OKButtonClick(TObject *Sender)
{
	bUpdateReg = TRUE;
   Close();
}
//---------------------------------------------------------------------------

void __fastcall TFormConf::CancelButtonClick(TObject *Sender)
{
	bUpdateReg = FALSE;
	Close();
}
//---------------------------------------------------------------------------


void __fastcall TFormConf::AdvancedButtonClick(TObject *Sender)
{
	FormAdvanced->ShowModal();
}
//---------------------------------------------------------------------------

void __fastcall TFormConf::mnuHelpContentsClick(TObject *Sender)
{
   // function in ESCUTIL: help_fns.cpp, opens help file and links to topic requested
   HelpLink("ServerConfig_Config.htm");
}
//---------------------------------------------------------------------------

void __fastcall TFormConf::mnuHelpAboutClick(TObject *Sender)
{
	AboutBox->ShowModal();
}
//---------------------------------------------------------------------------

void __fastcall TFormConf::efMaxLogFileSizeExit(TObject *Sender)
{
	// Check to make sure the value is valid
	if(ValidMaximumLogFileSize())
   {
   	maxLogFileSize = atoi(efMaxLogFileSize->Text.c_str());
   }
}
//---------------------------------------------------------------------------

bool __fastcall TFormConf::ValidMaximumLogFileSize(void)
{
	int value = efMaxLogFileSize->Text.ToIntDef(-1);

   if(value < 0)
   {
   	Application->MessageBox("Value must a valid integer >= 0",
      	"Maximum Log File Size", MB_OK | MB_ICONERROR);

      // Set to default
      efMaxLogFileSize->Text = "500";

      // Set cursor to efMaximumLogFileSize field
		FocusControl(efMaxLogFileSize);
      return(false);
   }

   return(true);
}
//---------------------------------------------------------------------------
void __fastcall TFormConf::cbUnlimitedLogFileSizeClick(TObject *Sender)
{
	efMaxLogFileSize->Enabled = !cbUnlimitedLogFileSize->Checked;
	efMaxLogFileSize->Color = cbUnlimitedLogFileSize->Checked ? clInactiveCaptionText : clWindow;
   if(cbUnlimitedLogFileSize->Checked)
   {
   	efMaxLogFileSize->Text = "";
   }
   else
   {
   	efMaxLogFileSize->Text = maxLogFileSize;
	}
}
//---------------------------------------------------------------------------
void __fastcall TFormConf::EditTALButtonClick(TObject *Sender)
{
   // Open TFLineList with file in efActiveTAL
   AnsiString TALFile = "\"";
   TALFile += efActiveTAL->Text.c_str();
   TALFile += "\"";
   spawnlp(P_NOWAIT, "TFLineList.exe", "TFLineList.exe", TALFile.c_str(), NULL);
}
//---------------------------------------------------------------------------

