#include <sstream>//for string conversions
#include <fstream>

#include "XMLTFAppParser.h"

XMLTFAppParser::XMLTFAppParser(string fileName,Log* outLog)
{
   ivLog = outLog;
   filePath = fileName;

   doc.SetCondenseWhiteSpace(0);//make sure that all whitespace gets conserved
   doc.LoadFile(fileName);
}

XMLTFAppParser::~XMLTFAppParser(void)
{          
} 

//Retrieves a node specified by step.A null pointer is returned if the 'step' argument is
//out of range.
TiXmlNode * XMLTFAppParser::GetNode(int step)
{
   return step < StepSize() ? VectorStepNodes.at(step) : NULL;
}

// Name: GetParamValue
// Purpose: Checks if the parameter value specified by paramIndex exists in the xml
//          file. If the parameter value exists it return the value by reference in
//          'string& value' if it doesn't exists it returns an empty 'string& value'.
// Note: - The parameter value is the 'Entry' attribute in the teleflow xml step.
//       - 'pParent' points to the node containing a teleflow step.
//       - 'paramIndex' specifies which parameter field. Must be between 1 and 9.
//       - 'value' is expected to be empty or will get overwritten.
int XMLTFAppParser::GetParamValue(TiXmlNode* pParent,int paramIndex, string& value)//should be protected.
{  
   int number;                  //Will hold the attribute value of 'Entry='
   string convertParamIndex;    //Will hold the converted integer value of 'number'
   TiXmlNode* pChild = 0;       //Points to the child nodes of pParent.
   TiXmlElement* pElement;      //Childnode will get

   if(pParent == NULL)
   {
      value.clear();
      return 0;
   }

   pChild = pParent->IterateChildren(pChild);   //Get The Child steps

   if(pChild == NULL)
   {
      value.clear(); 
      return 0;
   }

   while(pChild != NULL)
   {
      if(pChild->ValueStr() == "Parameter")
      {  
         pElement = pChild->ToElement(); //has to be an element

         if(pElement == NULL)
         {
            value.clear(); 
            return 0;
         }

         if(QueryXmlAttribute(pElement,"Entry", &number) && (number == paramIndex))
         {
            ConvertToString(paramIndex, convertParamIndex);
            value = convertParamIndex;
            return 1;
         }
      }
      pChild = pParent->IterateChildren(pChild);
   }

   value.clear();
   return 0;
}

// Name: GetParamDescription
// Purpose: Obtains the description parameter from the teleflow xml file. The description
//          parameter is the 'Description=' attribute part of the parameter field in
//          the xml file. If no description attribute is found an empty string 'value'
//          is returned by reference. Otherwise the Description is returned.
// Note:    -Method arguments are the same as method GetParamValue(...); defined above. 
int XMLTFAppParser::GetParamDescription(TiXmlNode* pParent, int paramIndex, string& value)
{
   int number;
   string data;
   string convertParamIndex;  

   TiXmlNode* pChild = NULL;
   TiXmlElement* pElement = NULL;
   const char* pData = NULL;

   if(pParent == NULL)
   {
      value.clear(); 
      return 0;
   }

   pChild = pParent->IterateChildren(pChild);   //Get The Child steps

   if(pChild == NULL)
   {
      value.clear(); 
      return 0;
   }

   while(pChild != NULL)
   {
      if(pChild->ValueStr() == "Parameter")
      {
         pElement = pChild->ToElement(); //has to be an element

         if(pElement == NULL)
         {
            value.clear();
            return 0;
         }

         if(QueryXmlAttribute(pElement,"Entry", &number) && number == paramIndex)
         {
           pData = pElement->Attribute("Description");
           value = pData;
           return 1;
         }
      }

      pChild = pParent->IterateChildren(pChild);
   }

   value.clear();
   return 0;
}

// Name: GetParamName
// Purpose: Obtains the name data from the parameter field in the teleflow xml.
//          The name data is either a variable or a constant it the field is empty
//          betweent he paramter tags an empty string is returned by reference.
// Note:    -Method arguments are the same as method GetParamValue(...); defined above. 
int XMLTFAppParser::GetParamName(TiXmlNode* pParent,int paramIndex,string& value)
{
   int number;                          //Will hold the parameter number read from file.
   TiXmlNode* pChild = NULL;            //Used to Iterate of all the childeren nodes from parent.
   TiXmlNode* pGrandChild = NULL;       //Holds the child node from 'pChild'.
   TiXmlElement* pElement = NULL;       //Use to convert the node to an Element node.
   TiXmlText* pText = NULL;             //Used to convert the node to a Text node.

   if(pParent == NULL)
   {
      value.clear(); 
      return 0;
   }
    //here is the error
   pChild = pParent->IterateChildren(pChild);   //Get The Child steps

   if(pChild == NULL)
   {
      value.clear();
      return 0;
   }

   while(pChild != NULL)
   {
      if(pChild->ValueStr() == "Parameter")
      {
         pElement = pChild->ToElement(); //has to be an element

         if(pElement == NULL)
         {  
            value.clear(); 
            return 0;
         }

         if(QueryXmlAttribute(pElement,"Entry", &number) && (number == paramIndex))
         {
            pGrandChild = pElement->FirstChild();

            if(pGrandChild == NULL)
            {
               value.clear(); 
               return 0;
            }

            pText = pGrandChild->ToText();

            if(pText == NULL)
            {
               value.clear();
               return 0;
            }
            
            value = pText->ValueStr();
            return 1; 
         }

       }

      pChild = pParent->IterateChildren(pChild);
   }

   value.clear();
   
   return 0;

}

// Name: GetParamType
// Purpose: Obtains the parameter type value from the parameter field. GetParamType
//          is the 'PassedBy=' attribute in the teleflow XML. If the variable is
//          passed by reference the return value for 'num' is a 1. If the variable
//          is passed by value the return value for 'num' is a 0.
int XMLTFAppParser::GetParamType(TiXmlNode* pParent, int paramIndex, int* num)
{
   int number;                   //Used to check for the proper parameter field. 
   string data;                  //Will hold the the attribute value from 'PassedBy='
   TiXmlNode* pChild = NULL;     //The child node from the 'pParent' Node.
   TiXmlElement* pElement;       //Used to convert to an Element Node.

   if(pParent == NULL)
   {
      *num = 0; 
      return 0;
   }

   pChild = pParent->IterateChildren(pChild);   //Get The Child steps

   if(pChild == NULL)
   {
      *num = 0;
      return 0;
   }

   while(pChild != NULL)
   {
      if(pChild->ValueStr() == "Parameter")
      {
         pElement = pChild->ToElement(); //has to be an element

         if(pElement == NULL)
         {
            *num = 0; 
            return 0;
         }

         if(QueryXmlAttribute(pElement,"Entry", &number) && (number == paramIndex))
         {
            QueryXmlAttribute(pElement,"PassedBy", &data);

            if(data == "Value")
            {
               *num = 0;
               return 1;
            }

            if(data == "Reference")
            {
               *num = 1;
               return 1;
            } 
            return 0;
         }
      }

      pChild = pParent->IterateChildren(pChild);
   }

   return 0;
}

// Name: QueryXmlAttribute
// Purpose: Retrieves from the node 'pElement' and the attribute specified by 'inputValue'
//          the value at the attribute.
// Note: -This method does not work properly when T is a string. 
template<class T>
int XMLTFAppParser::QueryXmlAttribute(TiXmlElement* pElement, string inputValue, T* returnValue)
{
   int errorCode;   
   errorCode = pElement->QueryValueAttribute(inputValue, returnValue );

   switch(errorCode)
   {
      case TIXML_SUCCESS:
      {
         break;
      }
      case TIXML_WRONG_TYPE:
      {
         return 0;
      }
      case TIXML_NO_ATTRIBUTE:
      {  
         return 0;
      }
      default: return 0;
   }

   return 1;
}

// Name: StepSize()
// Purpose: Returns the amount of steps in the tam or tap file.
int XMLTFAppParser::StepSize(void)
{
  return VectorStepNodes.size();
}

// Name: LoadXmlSteps()
// Purpose: Retrieves the root of the xml document and loads all step nodes into
//          a vector. 
int XMLTFAppParser::LoadXmlSteps(void)
{
   TiXmlElement* pElem = doc.RootElement();    //Obtains the root node.
    
   if(pElem == NULL)
   {
      return 0;
   }

   LoadVectorSteps(pElem); //has to return error or success. 
   return 1;
}

int XMLTFAppParser::Empty(void)
{
   return VectorStepNodes.empty(); // returns 1 on empty otherwise 0.
}

// Name:LoadVectorSteps
// Purpose: To load all the teleflow step nodes in the xml file into a vector for
//          easy retrieval.
// Note: -'TiXmlElement*' points to the root Node elemetn of the Document.
int XMLTFAppParser::LoadVectorSteps(TiXmlElement* pElem)
{
   TiXmlNode* pNxtSib; //a pointer to the next step node
   TiXmlNode* pChild;  //a pointer to the current step node

   if(pElem == NULL)
   {
      return 0; 
   }

   pChild = pElem->FirstChild();//Get the first Child Tag <Teleflow Step1>

   if(pChild == NULL)
   {
      return 0;
   }
   

   do
   {
      if(pChild->ValueStr() == "TeleFlowStep")
      {
          VectorStepNodes.push_back(pChild);
      }

   pNxtSib = pChild->NextSibling();//Get the next step
   pChild = pNxtSib;

   }
   while(pNxtSib != NULL);
   return 1; 
}


template<class T>
void XMLTFAppParser::ConvertToString(T source, string& destination)
{
   std::stringstream in;
   in << source;
   destination = in.str();
}

template<class T>
void XMLTFAppParser::ConvertToNumeric(string source, T* destination)
{
   std::stringstream outstream(source);
   outstream >> *destination;
}

// Name: GetFieldNumeric
// Purpose: Gets the value from any numeric field in a teleflow step. Numeric fields
//          are specified by '<n0x></n0x>'.
// Note: -'int step' specifies the particular teleflow step in the xml file.
//       -'in FieldIndex' specifies which numeric field to obtain the value from.
//          valid fieldIndex are from 1 till 8.
//       -'in *num' returns the value retrieved from the Numeric Field. If no
//          number is found, zero is returned. 
int XMLTFAppParser::GetFieldNumeric(int step,int fieldIndex, int* num)//working fine.
{
   TiXmlNode* pChild;                //holds the <teleflowstep>  node
   TiXmlNode* pGrandChilderen = 0;   //Pointer to the nodes from the child node
   TiXmlElement* pText;              //used to get text from a node
   TiXmlText* pData;                 //used to turn a node into a text node.
   string val;                       //holds the parsed field
   int convertedVal = 0;             //holds converted parsed field..
   string field;                     //holds the converted fieldIndex value
   string stringField = "n0";        //all fields start with element <n0x>

   if(step > StepSize())
   {
      return 0;
   }
   
   pChild = VectorStepNodes.at(step);

   ConvertToString(fieldIndex, field);
   stringField += field;//create the number variable field.

   pGrandChilderen = pChild->IterateChildren(pGrandChilderen); //Get the next node

   if(pGrandChilderen == NULL)//happens when there are no variables in the step.
   {
     *num = 0;//if variable not defined return 0; 
     return 1;
   }

   pData = pGrandChilderen->ToText(); //incase it is a <child step>data</child>

   if(pData != NULL && pGrandChilderen->ValueStr() == stringField)//check if the fields are equal and text is not Null. 
   {
      val = pData->ValueStr();
      ConvertToNumeric(val,&convertedVal);
      *num = convertedVal;

      return 1;
   }

   while(pGrandChilderen != NULL)
   {
     if(pGrandChilderen->ValueStr() == stringField)
     {
         pText = pGrandChilderen->ToElement();

         val = pText->GetText();
         ConvertToNumeric(val, &convertedVal);
         *num = convertedVal;

         return 1;
     }

    pGrandChilderen = pChild->IterateChildren(pGrandChilderen);
   }

   *num = 0;//default return value for Num
   return 1;
}

// Name: GetFieldString
// Purpose: To obtain the value from any String field in a teleflow xml file. String fields
//          are identified by '<s0x></s0x>' in the xml.
// Note: - 'fieldIndex' specifies any string field between 1 and 10.
//       - 'val' will contain the string value obtained or is empty if no string value is found. 
int XMLTFAppParser::GetFieldString(int step, int fieldIndex, string& val)//only returns zero when step is out of Range. 
{
   TiXmlNode* pChild;               //holds the <teleflowstep>
   TiXmlNode* pGrandChilderen = 0;  //holds the elements withing
   TiXmlElement* pText;
   TiXmlText* pData;
   string field;
   string stringField = "s";        //All string elements have sxx

   if(step > StepSize())
   {
      val.clear(); 
      return 0; 
   }
   
   pChild = VectorStepNodes.at(step);

   ConvertToString(fieldIndex, field);

   if(fieldIndex < 10)
   {
      stringField = stringField + "0" + field;//Append the field number
   }
   else
   {
      stringField += field;
   }

   pGrandChilderen = pChild->IterateChildren(pGrandChilderen); //Get the next node

   if(pGrandChilderen == NULL)//most likely happens when <child step></child>
   { 
     val.clear();
     return 1;
   }

   pData = pGrandChilderen->ToText(); //incase it is a <child step>data</child>

   if(pData != NULL && pGrandChilderen->ValueStr() == stringField)
   {
      val = pData->ValueStr();
      return 1;
   }

   while(pGrandChilderen != NULL)
   {
     if(pGrandChilderen->ValueStr() == stringField)
     {
         pText = pGrandChilderen->ToElement();
         val = pText->GetText();
         return 1;
     }
     
    pGrandChilderen = pChild->IterateChildren(pGrandChilderen);
   }
    
   val.clear(); 
   return 1;
}
// Name: GetRecordString
// Purpose: To retrieve any string attribute value from the '<TeleflowStep>' field.
// Note: - 'int step' specifies which step to a record from.
//       - 'string record' specifies the attribute name.
//       - 'string val' contains the attribute value retrieved. If the 'record' is
//          not found an empty string is returned. 
int XMLTFAppParser::GetRecordString(int step, string record, string& val)
{
   TiXmlNode* pChild =0;      //Step node
   TiXmlElement* pStep;       //Turn node in to Element to Querry Attributes
   const char* pName = NULL;  //Points the record obtained. 

   if(step > StepSize())
   {
      val.clear(); 
      return 0;
   }

   pChild = VectorStepNodes.at(step);

   pStep = pChild->ToElement();

   if(pStep == NULL)
   {
      val.clear();
      return 1;
   }
   
   pName = pStep->Attribute(record.c_str());

   if(pName == NULL)
   {
      val.clear();
      return 1;
   }

   val = pName;
   
   return 1; 
}

// Name: ConverToMyChar
// Purpose: To convert a type string to a type MyChar.
// Note: - The 'char *' pointer of 'destination' should be freed when finished. 
void XMLTFAppParser::ConvertToMyChar(string source, MYCHAR& destination)
{
   destination.arr = new char[source.length() + 1]; 
   strcpy(destination.arr, source.c_str());
   destination.len = source.length();
}

// Name: GetRecordNumeric
// Purpose: Obtains any numeric attribute value from a '<teleflowstep>' field.
// Note: - 'int step' specifies the specific teleflow step
//       - 'string record' specifies the attribute name.
//       - 'int *num' will contain the found attribute value. If non is found num
//          will equal zero. 
int XMLTFAppParser::GetRecordNumeric(int step, string record, int* num)//Is working fine
{
   TiXmlNode* pChild = 0;     //Step node
   TiXmlElement* pStep;       //Turn node in to Element to Querry Attributes
   int ErrorCheck;            //Return value to check if record is found
   string data;               //incase it is a Next record and not of the type int.
   const char* pData= NULL;   //Will point to the data allocated. 

   if(step > StepSize())
   {
      *num = 0;
      return 0;
   }

   pChild = VectorStepNodes.at(step);

   pStep = pChild->ToElement();

   ErrorCheck = pStep->QueryValueAttribute(record,num);

   switch(ErrorCheck)
   {
      case TIXML_SUCCESS:
      {
         break;
      }
      case TIXML_WRONG_TYPE:
      {
         if(record == "Next")//if it is a next Record it can have a string value.
         {
           pData = pStep->Attribute(record.c_str());//Has to be right type

           if(pData == NULL)
           {
               *num = -1;
               return 1;
           }
           data = pData;

           if(data == "Dead")
           {
               *num = -6;
               return 1;
           }
           else if(data == "Hidden")
           {
               *num = -5;
               return 1;
           }
           else if(data == "Unconnectable")
           {
               *num = -4;
               return 1;
           }
           else if(data == "Disconnected")
           {
               *num = 0;
               return 1;
           }
           else
           {
               *num = -1; //not found
               return 1;
           }
         }
         return 0;
      }
      case TIXML_NO_ATTRIBUTE:
      {   
         *num = 0; //Default value is zero even if it doesnt exist
         return 1;
      }
      default: return 0;
   }                  
   return 1;
}

