#include <sstream>//for string conversions
#include <fstream>

#include "XmlVoiceDBF.h"

XmlVoiceDBF::XmlVoiceDBF(string fileName)
{
   TiXmlElement* pElement; 
   doc.SetCondenseWhiteSpace(0);//Preserve WhiteSpace.
   
   doc.LoadFile(fileName);

   pElement = doc.RootElement();  //Check if this works if no file is found if not put in other member function and return Failure. 
   LoadEntries(pElement); 
}

XmlVoiceDBF::~XmlVoiceDBF()
{
}

// Name: GetVoxConv
// Purpose: Retrieves the "VoxCov=" attribute value from the xml file. 'int entry' specifies which voicescript entry
//          to retrieve from.
// Note: The "VoxConv=" attribute value can only be an integer value.
int XmlVoiceDBF::GetVoxConv(int entry, int* name)
{
   if(ParseRecordNumeric(entry, "VoxConv", name))
   {
      return 0;
   }
   else
   {
      return 1;
   }
}

// Name: GetWavFile
// Purpose: Retrieves the "WavFile=" attribute value from the xml file.
int XmlVoiceDBF::GetWavFile(int entry, string& name)
{
   if(ParseRecordString(entry, "WavFile",name) == 0)
   {
      return 0;
   }
   else
   {
      return 1;
   }
}

// Name: GetVoxFile
// Purpose: Retrieves the "VoxFile=" attribute value from the xml file. 
int XmlVoiceDBF::GetVoxFile(int entry, string& name)
{
   if(ParseRecordString(entry, "VoxFile", name) == 0)
   {
      return 0;
   }
   else
   {
      return 1;
   }
}

// Name: GetName
// Purpose: Retrieves the "GetName=" attribute value from the xml file. 
int XmlVoiceDBF::GetName(int entry, string& name)
{
   if(ParseRecordString(entry, "Name",name) == 0)
   {   
      return 0;
   }
   else
   {
      return 1;
   }
}

// Name: ParseRecordString
// Purpose: Parses any given attribute value specified by the attribute name 'string input'.
// Note: On Failure 'string output' will be empty and zero is returned. Otherwise the Attribute value
//       is returned in 'strint output' and one is returned.  
int XmlVoiceDBF::ParseRecordString(int entry, string input, string& output)
{
   TiXmlNode* pChild =0;      //Step node
   TiXmlElement* pStep;       //Turn node in to Element to Querry Attributes
   
   const char* pName = NULL;  //Points the record obtained. 

   if(entry >= Size())
   {
      output.clear(); 
      return 0;
   }

   pChild = XmlVoiceEntries.at(entry);

   pStep = pChild->ToElement();

   if(pStep == NULL)
   {
      output.clear();
      return 1;
   }
   
   pName = pStep->Attribute(input.c_str());

   if(pName == NULL)
   {
      output.clear();
      return 1;
   }

   output = pName;

   return 1;
}

// Name: ParseRecordNumeric 
// Purpose: Parse any Numeric attribute value specified by 'string output'
// Note: If no attribute value is found zero is returned otherwise 1 is returned. 
int XmlVoiceDBF::ParseRecordNumeric(int entry, string input, int* output)
{
   TiXmlNode* pChild = 0;       //Step node
   TiXmlElement* pEntry;        //Turn node in to Element to Querry Attributes
   int ErrorCheck;              //Return value to check if record is found

   if(entry >= Size())
   {
      *output = 0;
      return 0;
   }
   
   pChild = XmlVoiceEntries.at(entry);

   pEntry = pChild->ToElement();

   ErrorCheck = pEntry->QueryValueAttribute(input,output);

   switch(ErrorCheck)
   {
      case TIXML_SUCCESS:
      {
         break;
      }
      case TIXML_WRONG_TYPE:
      {
         *output = 0;
         return 0; 
      }
      case TIXML_NO_ATTRIBUTE:
      {   
         *output = 0; //Default value is zero even if it doesnt exist
         return 1;
      }
      default:
      {
         *output = 0;
         return 0;
      }
   }
                    
   return 1;
}

//Name: LoadEntries
//Purpose: Loads the Xml nodes that are a <TeleflowVoiceScript> into a vector for easy retrieval.
int XmlVoiceDBF::LoadEntries(TiXmlElement* pElement)
{
   TiXmlNode* pChild;
   TiXmlNode* pNxtSib;
   
   if(pElement == NULL)
   {
      return 0;
   }

   if(pElement->ValueStr() == "TeleFlowVoiceScripts")
   {
      isXml = true;
   }
   else
   {
      isXml = false;
      return 0; 
   }

   pChild = pElement->FirstChild();//Get the first Child Tag <TeleFlowVoiceScript>

   if(pChild == NULL)
   {
      return 0;
   }    

   do
   {
      if(pChild->ValueStr() == "TeleFlowVoiceScript")
      {
          XmlVoiceEntries.push_back(pChild);
      }

      pNxtSib = pChild->NextSibling();//Get the next step
      pChild = pNxtSib;

   }
   while(pNxtSib != NULL);

   return 1; 
} 

// Name: Size
// Purpose: Returns the number of entries in the Voice File.
int XmlVoiceDBF::Size(void)
{
   return XmlVoiceEntries.size();
}

// Name: IsXmlFile
// Purpose: Returns 1 or 0 when the file is a valid or invalid xml file. 
int XmlVoiceDBF::IsXmlFile(void)
{
   return isXml;
}
