#include <string>
using std::string;

#include "iv_math2.h"
#include "step_Case.h"

StepCase::StepCase(STEPREC& stepRecord) : StepTones(stepRecord), cases(9)
{
   cases[0] = stepRecord.s1.arr;
   cases[1] = stepRecord.s2.arr;
   cases[2] = stepRecord.s3.arr;
   cases[3] = stepRecord.s4.arr;
   cases[4] = stepRecord.s5.arr;
   cases[5] = stepRecord.s6.arr;
   cases[6] = stepRecord.s7.arr;
   cases[7] = stepRecord.s8.arr;
   cases[8] = stepRecord.s9.arr;
}

StepCase::~StepCase(void)
{
}

const char* StepCase::GetStepName(void) const
{
	return("Case");
}

IV_STATUS StepCase::Execute(PIVRINFO pInfo)
{
   char* value;
   bool bParmsValid = true;
   string parsedArgument;
   string parsedCaseValue;
   unsigned int caseNumber;
   char caseValueString[13];
   vector<string> parsedCases(9);
   IV_STATUS status;
   bool result = false;

   if(Status(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FAIL);
   }

   if((value = pInfo->variables->ParseParameter("Case argument", cases[0].c_str(), "")) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      parsedArgument = value;
      delete[] value;
   }

   strcpy(caseValueString, "Case 0 value");

   for(caseNumber = 1; caseNumber <= 8; caseNumber++)
   {
      caseValueString[5] = (char) ('0' + caseNumber);

      if((value = pInfo->variables->ParseParameter(caseValueString, cases[caseNumber].c_str(), "")) == NULL)
      {
         bParmsValid = false;
      }
      else
      {
         parsedCases[caseNumber] = value;
         delete[] value;
      }
   }

   if(!bParmsValid)
   {
      return(IVR_FATAL);
   }

   for(caseNumber = 1; (caseNumber <= 8) && !result; caseNumber++)
   {
      if((status = VarCmp(pInfo, parsedArgument.c_str(), parsedCases[caseNumber].c_str(), FN_EQ, result, true)) != IVR_SUCCESS)
      {
         return(status);
      }
   }

   if(!result)
   {
      pInfo->ivLog->Print("      No case found\n");
      SetNextStep(GetTTOther());
   }
   else
   {
      SetNextStep(GetTT(caseNumber - 1));
   }

   return(IVR_SUCCESS);
}
