#include <stdlib.h>               // atoi
#include "step_calltrigger.h"
#include "..\escutil\iv_str.h"	// strtrim
#include "..\BoardDLLs\brd_generic.h"

StepCallTrigger::StepCallTrigger(STEPREC& stepRecord) : Step(stepRecord)
{
   triggerList = stepRecord.s2.arr;
   triggerVar = stepRecord.s3.arr;
   triggerInfoVar = stepRecord.s5.arr;
   timeout = stepRecord.s4.arr;
}

StepCallTrigger::~StepCallTrigger(void)
{
}

const char* StepCallTrigger::GetStepName(void) const
{
	return("Call Trigger");
}

IV_STATUS StepCallTrigger::Execute(PIVRINFO pInfo)
{
   IV_STATUS status;
   // Evaluate all parameters before exiting
   bool bParmsValid = true;
   long iTimeout;
   char* value;
   string parsedTriggerList;
   const char* trigger;
   char* triggerInfoStr = NULL;

   if(Status(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FAIL);
   }

	if(PortRequired(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FATAL);
   }

   if((value = pInfo->variables->ParseParameter("Trigger list", triggerList.c_str())) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      parsedTriggerList = value;
      delete[] value;
   }

   // Note that FOREVER == -1
   if((value = pInfo->variables->ParseParameter("TIMEOUT", timeout.c_str(),
      "-1", "Empty TIMEOUT defaults to none")) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      iTimeout = atoi(value);  // iTimeout is in milliseconds.
      delete[] value;

      if((iTimeout < 0) && (iTimeout != FOREVER))
      {
		   pInfo->ivLog->Print("      ERROR: TIMEOUT '%s' must be >= 0\n", timeout.c_str());
         bParmsValid = false;
      }
   }

   if(!bParmsValid)
   {
      return(IVR_FATAL);
   }
   
   status = pInfo->ivBoard->CallTrigger(parsedTriggerList.c_str(), trigger, triggerInfoStr, iTimeout);

   if(status == IVR_SUCCESS)
   {
      if(!triggerVar.empty() && !pInfo->variables->Set(triggerVar.c_str(), trigger))
      {
         pInfo->ivLog->Print("      ERROR: Failed to set Trigger Event variable '%s'\n", triggerVar.c_str());
         status = IVR_FATAL;
      }

      if(!triggerInfoVar.empty() && !pInfo->variables->Set(triggerInfoVar.c_str(), triggerInfoStr))
      {
         pInfo->ivLog->Print("      ERROR: Failed to set Additional Info variable '%s'\n", triggerInfoVar.c_str());
         status = IVR_FATAL;
      }
   }

	delete[] triggerInfoStr;

   return(status);
}
