#pragma hdrstop
#include "..\escutil\security.h"
#include "..\BoardDLLs\brd_generic.h"
#include "step_encrypt.h"

StepEncrypt::StepEncrypt(STEPREC& stepRecord) : Step(stepRecord)
{
   resultVar = stepRecord.s3.arr;
   text = stepRecord.s1.arr;
   key = stepRecord.s2.arr;
   uppercase = (bool) stepRecord.u1;
}

StepEncrypt::~StepEncrypt(void)
{
}

const char* StepEncrypt::GetStepName(void) const
{
	return("Encrypt");
}

IV_STATUS StepEncrypt::Execute(PIVRINFO pInfo)
{
   char* value;
   // Evaluate all parameters before exiting
   bool bParmsValid = true;
   string parsedText;
   string parsedKey;
   char* pszResult;
   IV_STATUS status;

   if(Status(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FAIL);
   }

   if((value = pInfo->variables->ParseParameter("Plain text", text.c_str(), "")) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      parsedText = value;
      delete[] value;
   }

   if((value = pInfo->variables->ParseParameter("Encryption key", key.c_str(), "")) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      parsedKey = value;
      delete[] value;
   }

   if(!bParmsValid)
   {
      return(IVR_FATAL);
   }

   if((pszResult = Encrypt(parsedText.c_str(), parsedKey.c_str(), PKCS_5_PADDING, uppercase)) == NULL)
   {
      return(IVR_FATAL);
   }

   status = pInfo->variables->Set(resultVar.c_str(), pszResult) ? IVR_SUCCESS : IVR_FATAL;

   delete[] pszResult;

   return(status);
 }
