#include "step_message.h"

StepMessage::StepMessage(STEPREC& stepRecord) : Step(stepRecord)
{
   fileNm = stepRecord.s1.arr;
   msg = stepRecord.s6.arr;
   bNewLine = (bool) stepRecord.u1;
   bAppend = (bool) stepRecord.u2;
}

StepMessage::~StepMessage(void)
{
}

const char* StepMessage::GetStepName(void) const
{
	return("Write Text");
}

// -------------------------------------------------------------------------
//   Name:        StepMessage::Execute
//
//   Purpose:     Starts a new application (as opposed to "call").  Does not
//                add on to the LoopQ return stack.
//
//   Questions:   - How does this work?  A TAP file loads all
// -------------------------------------------------------------------------
IV_STATUS StepMessage::Execute(PIVRINFO pInfo)
{
   char* value;
   // Evaluate all parameters before exiting
   bool bParmsValid = true;
   string parsedMessage;

   if(Status(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FAIL);
   }

   if((value = pInfo->variables->ParseParameter("Message", msg.c_str(), "")) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      parsedMessage = value;
      delete[] value;
   }

   if(!fileNm.empty())
   {
      FILE* fout;
      string parsedFileName;

      if((value = pInfo->variables->ParseParameter("File", fileNm.c_str())) == NULL)
      {
         bParmsValid = false;
      }
      else
      {
         parsedFileName = value;
         delete[] value;

         parsedFileName = value = pInfo->variables->FilePath(parsedFileName.c_str());
         delete[] value;

         pInfo->ivLog->Print("      File path is '%s'\n", parsedFileName.c_str());
      }

      if(!bParmsValid)
      {
         return(IVR_FATAL);
      }

      if((fout = fopen(parsedFileName.c_str(), bAppend ? "a" : "w")) == NULL)
      {
         pInfo->ivLog->Print("      ERROR: Message could not open file '%s'.\n", parsedFileName.c_str());
         return(IVR_FAIL);
      }

      if(!parsedMessage.empty())
      {
         fprintf(fout, "%s", parsedMessage.c_str());
      }

      if(bNewLine)
      {
         fprintf(fout, "\n");
      }

      fclose(fout);
   }
   else if(!bParmsValid)
   {
      return(IVR_FATAL);
   }
   else
   {
      if(!parsedMessage.empty())
      {
         pInfo->ivLog->Print("%s", parsedMessage.c_str());
      }

      if(bNewLine)
      {
         pInfo->ivLog->Print("\n");
      }
   }

   return(IVR_SUCCESS);
}
