#include "..\escutil\iv_str.h"	// strtrim
#include "..\escutil\odbc_db.h"
#include "initglobals.h"         // InitializeGlobals
#include "..\xmlutil\xmllist.h"
#include "VCE_List.h"
#include "step_runtap.h"
#include "LoadLanguage.h"

StepRunTAP::StepRunTAP(STEPREC& stepRecord) : Step(stepRecord)
{
   TAPName = stepRecord.s2.arr;
   parmList = stepRecord.s6.arr;
   maintainSQLList = (bool) stepRecord.u1;
}

StepRunTAP::~StepRunTAP(void)
{
}

const char* StepRunTAP::GetStepName(void) const
{
	return("Run TAP");
}

IV_STATUS StepRunTAP::Execute(PIVRINFO pInfo)
{
   IV_STATUS status;
   // Save old information, note that this is a deliberately shallow copy!
   IVRINFO oldInfo = *pInfo;
   char parmName[8];
   int index;
   const char* start = parmList.c_str();
   const char* end;
   char* parsedTAPName;
   char* value;
   char languageDLL[1024];

   if(Status(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FAIL);
   }

   if((pInfo->pszAppFile = pInfo->variables->ParseParameter("TAP name", TAPName.c_str())) == NULL)
   {
      // Restore previous pInfo structure
      *pInfo = oldInfo;
      return(IVR_FATAL);
   }

   strtrim(pInfo->pszAppFile);
   value = pInfo->variables->FilePath(pInfo->pszAppFile);
   delete[] pInfo->pszAppFile;
   pInfo->pszAppFile = value;

   parsedTAPName = strrchr(pInfo->pszAppFile, '\\');
   // If there is no slash use the whole name, if there is, increment position
   parsedTAPName = (parsedTAPName == NULL) ? pInfo->pszAppFile : parsedTAPName + 1;

   pInfo->variables = new VarController(pInfo);
   InitializeGlobals(pInfo);
   pInfo->voiceList = new VoiceList();
   if(!maintainSQLList)
   {
      pInfo->odbcManager = new ODBCManager(pInfo);
   }
   pInfo->xmlList = new XMLList(pInfo->ivLog);

   // Get language module DLL name so that it can be re-loaded
   GetModuleFileName(pInfo->languageLibrary, languageDLL, 1024);
	pInfo->ivLog->Print("   Re-loading language module '%s'\n", languageDLL);

   // Clear current values so that the current library doesn't get unloaded
   pInfo->languageLibrary = NULL;
   pInfo->language = NULL;

   // Reload language module
	if(!LoadLanguageModule(pInfo, languageDLL))
   {
		pInfo->ivLog->Print("   ERROR: Could not carry forward language module\n");
      // Free memory used to this point
	   delete[] pInfo->pszAppFile;
	   delete pInfo->variables;
	   delete pInfo->voiceList;
      if(!maintainSQLList)
      {
	      delete pInfo->odbcManager;
      }
	   delete pInfo->xmlList;
      // Restore previous pInfo structure
      *pInfo = oldInfo;
      // Generate fatal error
      return(IVR_FATAL);
   }

   // Carry forward the @PROMPT global variable if it exists
   pInfo->variables->SetSystem("@PROMPT", oldInfo.variables->Get("@PROMPT"));

   // Carry forward the @TIMEOUT global variable if it exists
   pInfo->variables->SetGlobal("@TIMEOUT", oldInfo.variables->Get("@TIMEOUT"));

   // Carry forward the @VOLUME global variable if it exists
   pInfo->variables->SetGlobal("@VOLUME", oldInfo.variables->Get("@VOLUME"));

   // Carry forward the @SYS_CALLREF system variable if it exists
   pInfo->variables->SetSystem("@SYS_CALLREF", oldInfo.variables->Get("@SYS_CALLREF"));

   // Carry forward the @CALLOUT_RESULT system variable if it exists
   pInfo->variables->SetSystem("@CALLOUT_RESULT", oldInfo.variables->Get("@CALLOUT_RESULT"));

   // Carry forward the @SYS_CALL_DIRECTION system variable if it exists
   pInfo->variables->SetSystem("@SYS_CALL_DIRECTION", oldInfo.variables->Get("@SYS_CALL_DIRECTION"));

   // Carry forward the @SYS_DATA system variable if it exists
   pInfo->variables->SetGlobal("@SYS_DATA", oldInfo.variables->Get("@SYS_DATA"));

   // Create system variables for parameters (@P1 - @P9)
   for(index = 0; (start != NULL) && (start[0] != '\0') && (index < 9); index++)
   {
      end = strchr(start, END_SUBFIELD);
      pInfo->pszParams[index] = new char[end - start + 1];
      strncpy(pInfo->pszParams[index], start, end - start);
      pInfo->pszParams[index][end - start] = '\0';
      sprintf(parmName, "@P%d", index + 1);
      if(oldInfo.variables->IsParsedInvalid(pInfo->pszParams[index]))
      {
         pInfo->ivLog->Print("      ERROR: Parameter @P%d '%s' is undefined\n", index + 1, pInfo->pszParams[index]);
         // Restore previous pInfo structure
         *pInfo = oldInfo;
         return(IVR_FATAL);
      }
      value = oldInfo.variables->Parse(pInfo->pszParams[index]);
      pInfo->variables->SetSystem(parmName, value);
      delete[] value;
      start = end + 2;
   }
   for(; index < 9; index++)
   {
      pInfo->pszParams[index] = new char[1];
      pInfo->pszParams[index][0] = '\0';
      sprintf(parmName, "@P%d", index + 1);
      pInfo->variables->SetSystem(parmName, pInfo->pszParams[index]);
   }

   pInfo->ivLog->Print("   %s through database connections and handles\n", maintainSQLList ? "Passing" : "Not passing");

   pInfo->ivLog->Print("'%s'==>'%s'\n", oldInfo.pszAppFile, pInfo->pszAppFile);

   pInfo->loadingMutex->Get();
   TAP* tap = new TAP(pInfo);
   pInfo->loadingMutex->Release();

   status = RunTAPSEH(pInfo, &oldInfo, tap);

   // Unload current language module
   UnloadLanguageModule(pInfo);

   pInfo->ivLog->Print("'%s'<=='%s'\n", oldInfo.pszAppFile, pInfo->pszAppFile);

   delete tap;

   for(index = 0; index < 9; index++)
   {
      delete[] pInfo->pszParams[index];
   }

   delete[] pInfo->pszAppFile;
   delete pInfo->voiceList;
   if(!maintainSQLList)
   {
      delete pInfo->odbcManager;
   }
   // Must delete the VarController *after* the ODBC manager;
   // ODBC Manager destructor *uses* variables!
   delete pInfo->variables;
   delete pInfo->xmlList;

   if(pInfo->status == lsCrashed)
   {
      status = IVR_FATAL;
   }

   // Preserve changes to restart/reload/terminate flags except safeToShutdown
   oldInfo.bRestart = pInfo->bRestart;
   oldInfo.bTerminate = pInfo->bTerminate;
   oldInfo.reloadRequested = pInfo->reloadRequested;
   oldInfo.shutdownRequested = pInfo->shutdownRequested;

   // Preserve changes to Voice libraries/modules
   oldInfo.speechRecognitionLibrary = pInfo->speechRecognitionLibrary;
   oldInfo.speechRecognition = pInfo->speechRecognition;
   oldInfo.voiceAuthenticationLibrary = pInfo->voiceAuthenticationLibrary;
   oldInfo.voiceAuthentication = pInfo->voiceAuthentication;
   oldInfo.textToSpeechLibrary = pInfo->textToSpeechLibrary;
   oldInfo.textToSpeech = pInfo->textToSpeech;

   // Restore old information, note that this is a deliberately shallow copy!
   *pInfo = oldInfo;

   // Restore creation/destruction functions of language module
   if(!LoadLanguageModuleFunctions(pInfo->languageLibrary))
   {
      pInfo->ivLog->Print("   ERROR: Unable to reload language module functions\n");
      return(IVR_FAIL);
   }

   return((status == IVR_ENDPROC) ? IVR_SUCCESS : status);
}
/*StepRunTAP::Execute*/

IV_STATUS StepRunTAP::RunTAP(PIVRINFO pInfo, PIVRINFO pOldInfo, TAP* tap)
{
   IV_STATUS status;

   try
   {
      if((status = tap->LoadParameters(pInfo)) == IVR_SUCCESS)
      {
         if(tap->GetReady())
         {
            status = tap->Execute(pInfo);
            status = tap->SetReferenceParameters(pInfo, pOldInfo, parmList.c_str()) != IVR_SUCCESS ? IVR_FATAL : status;
         }
         else
         {
            status = IVR_FAIL;
         }
      }
   }
   // Catch STL base exception, report what string, and rethrow to be caught in standard exception handler
   catch (std::exception stlException)
   {
      pInfo->ivLog->Print("STLException: '%s'\n", stlException.what());
      throw;
   }

   return status;
}

IV_STATUS StepRunTAP::RunTAPSEH(PIVRINFO pInfo, PIVRINFO pOldInfo, TAP* tap)
{
   EXCEPTION_POINTERS* xp;

   __try
   {
      return RunTAP(pInfo, pOldInfo, tap);
   }
   __except(xp = GetExceptionInformation(), EXCEPTION_EXECUTE_HANDLER)
   {
      unsigned int xc = GetExceptionCode();
      EXCEPTION_RECORD* xr = xp->ExceptionRecord;

      if(xc == EXCEPTION_ACCESS_VIOLATION)
      {
         pInfo->ivLog->Print("%lX: EXCEPTION_ACCESS_VIOLATION attempting to %s at location %lX.\n",
            xr->ExceptionAddress, xr->ExceptionInformation[0] ? "write" : "read",
            xr->ExceptionInformation[1]);
      }
      else
      {
         pInfo->ivLog->Print ("EXCEPTION CODE %x.\n", xc);
      }

      pInfo->ivLog->Print("Exception occurred!  TAP exiting.\n");
      return(IVR_FATAL);
   }
}