#include "..\escutil\odbc_db.h"
#include "step_sqlconnect.h"

StepSQLConnect::StepSQLConnect(STEPREC& stepRecord) : Step(stepRecord)
{
   connectParms = stepRecord.s6.arr;
   connectionHandle = stepRecord.s2.arr;
}

StepSQLConnect::~StepSQLConnect(void)
{
}

const char* StepSQLConnect::GetStepName(void) const
{
	return("DB Connect");
}

IV_STATUS StepSQLConnect::Execute(PIVRINFO pInfo)
{
   IV_STATUS  status;
   char* valueParms = NULL;
   char* valueHandle = NULL;
   bool bParmsValid = true;
   SQLList* pSQLConnection = NULL;

   if(Status(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FAIL);
   }

   // WAS:   if(pInfo->sqlList == NULL)
   if(pInfo->odbcManager == NULL)
   {
      pInfo->ivLog->Print("      ERROR: SQL not initialized.\n");
      return(IVR_FATAL);
   }

   if((valueParms = pInfo->variables->ParseParameter("Connect Parameters", connectParms.c_str())) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      pInfo->ivLog->Print("      Connect String: %s\n", valueParms);
   }

   if((valueHandle = pInfo->variables->ParseParameter("Connection Handle", connectionHandle.c_str(), "")) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      if(strcmp(valueHandle,"") == 0)
      {
         pInfo->ivLog->Print("      Using default ODBC handle\n");
      }
   }

   if(!bParmsValid)
   {
      delete[] valueParms;    // safe to delete, pointers were initialized to NULL
      return(IVR_FATAL);
   }

   // Check if a connection by that handle already exists
   if((pSQLConnection = pInfo->odbcManager->FindConnection(valueHandle)) != NULL)
   {
      // Check if the connection parameters were the same
      if(pSQLConnection->ConnectionStringMatch(valueParms))
      {
         pInfo->ivLog->Print("      Handle already connected.\n");
         status = IVR_SUCCESS;
      }
      else
      {
         pInfo->ivLog->Print("      ERROR: Handle already in use.\n");
         status = IVR_FATAL;
      }
   }
   else
   {
      // Connect through ODBC connection manager (which will store the new connection)
      status = pInfo->odbcManager->Connect(valueHandle, valueParms);
   }

   delete[] valueHandle;
   delete[] valueParms;

   return(status);
}
