//---------------------------------------------------------------------------

#ifndef TFSimulator_mainH
#define TFSimulator_mainH
//---------------------------------------------------------------------------
#include "..\TFServer\ivrinfo.h"

#include <Classes.hpp>
#include <Controls.hpp>
#include <StdCtrls.hpp>
#include <Forms.hpp>

#include "..\TFServer\TFServer_communication.h"
#include "..\TFServer\TFEngine_communication_manager.h"
#include "..\BoardDLLs\brd_simulator_communication.h"
#include "TFSimulator_communication.h"
#include "..\TFServer\gui_log.h"           	// GUILog
#include "msg_win.h"                         // TMsgWindow
#include "..\TFServer\TFApplicationsManager.h"
#include "..\TFServer\TFServerRegistrationInfo.h"

#include <mmsystem.h>

#include <ExtCtrls.hpp>
#include <Graphics.hpp>
#include <ImgList.hpp>
#include <Dialogs.hpp>
#include <Menus.hpp>
#include <Buttons.hpp>
#include <ComCtrls.hpp>
#include <Tabnotbk.hpp>
#include <FileCtrl.hpp>
#include <jpeg.hpp>
//---------------------------------------------------------------------------

// Registry key where TeleFlow Monitor settings are stored
#define TFSIMULATOR_REG_KEY  TELEFLOW_REG_KEY "\\TeleFlow Simulator"

// Need a forward declaration for 'TFormSimulator'
// to prevent a circular reference.
class TFormSimulator;

class SimLog : public GUILog
{
   public:

      SimLog(TFormSimulator* simulator, int maxFileSize, unsigned int maxLines = 100,
         unsigned int maxLineSize = 100);
      virtual ~SimLog(void);

      void PrintLine(const std::string& asLine, bool bEOL);

      void Deactivate(void);

   private:

      Mutex mutex;
      TFormSimulator* simulator;
};

typedef struct MonitorParameters
{
   HANDLE waitObjs[3];
   TFormSimulator* simulator;
   TFSimulatorCommunicationDestination* simulatorMessages;
   TFServerCommunicationDestination* serverMessages;
} MonitorParameters;

typedef struct CallSystemParameters
{
   TFormSimulator* simulator;
   int appID;
   HMODULE library;
   HANDLE shutdown;
} CallSystemParameters;

class TFormSimulator : public TForm
{
__published:	// IDE-managed Components
   TMainMenu *MainMenu1;
   TMenuItem *FileMenu;
   TMenuItem *FileMenuOpen;
   TMenuItem *FileMenuExit;
   TMenuItem *SimulationMenu;
   TMenuItem *SimulationMenuStart;
   TMenuItem *SimulationMenuPause;
   TMenuItem *HelpMenu;
   TMenuItem *HelpMenuHelp;
   TMenuItem *HelpMenuAbout;
   TOpenDialog *OpenDialog;
   TMenuItem *FileMenuSetLogDirectory;
   TMenuItem *LogMenu;
   TMenuItem *LogMenuLogDirectory;
   TStatusBar *StatusBar;
   TPanel *ApplicationPanel;
   TImage *NormalStatusLightImage;
   TLabel *StatusLabel;
   TMenuItem *LogMenuMaximumShortPlay;
   TGroupBox *StatusGroupBox;
   TMemo *StatusMemo;
   TMenuItem *LogMenuFreezeDisplayAtError;
   TPanel *KeypadPanel;
   TImage *Button1DownImage;
   TImage *Button2DownImage;
   TImage *Button3DownImage;
   TImage *Button4DownImage;
   TImage *Button5DownImage;
   TImage *Button6DownImage;
   TImage *Button7DownImage;
   TImage *Button8DownImage;
   TImage *Button9DownImage;
   TImage *ButtonStarDownImage;
   TImage *Button0DownImage;
   TImage *ButtonPoundDownImage;
   TImage *Button1Image;
   TImage *Button2Image;
   TImage *Button3Image;
   TImage *Button4Image;
   TImage *Button5Image;
   TImage *Button6Image;
   TImage *Button7Image;
   TImage *Button8Image;
   TImage *Button9Image;
   TImage *ButtonStarImage;
   TImage *Button0Image;
   TImage *ButtonPoundImage;
   TPanel *PhonePanel;
   TImage *OffHookImage;
   TPanel *CallHangupPanel;
   TImage *CallSystemImage;
   TImage *CallSystemDownImage;
   TImage *CallSystemDisabledImage;
   TImage *HangupImage;
   TImage *HangupDownImage;
   TImage *HangupDisabledImage;
   TPanel *StopRecordingPanel;
   TImage *StopRecordingImage;
   TImage *StopRecordingDownImage;
   TImage *StopRecordingDisabledImage;
   TImage *KeypadPanelImage;
   TImage *OnHookImage;
   TPanel *ButtonPanel;
   TImage *ButtonPanelImage;
   TPanel *PauseResumePanel;
   TImage *PauseImage;
   TImage *PauseDownImage;
   TImage *ResumeImage;
   TImage *ResumeDownImage;
   TImage *ResumeDisabledImage;
   TImage *PauseDisabledImage;
   TPanel *NextStepPanel;
   TImage *NextStepImage;
   TImage *NextStepDownImage;
   TImage *NextStepDisabledImage;
   TPanel *ShowLogPanel;
   TImage *ShowLogImage;
   TImage *ShowLogDownImage;
   TImage *StopRecordingBlankImage;
   TImage *NextStepBlankImage;
   TImage *CrashedStatusLightImage;
   TImage *NotRunningStatusLightImage;
   TImage *BargeInImage;
   TImage *BargeInDownImage;
   TImage *BargeInDisabledImage;
   void __fastcall FileMenuOpenClick(TObject *Sender);
   void __fastcall SimulationMenuStartClick(TObject *Sender);
   void __fastcall LogMenuMaximumShortPlayClick(TObject *Sender);
   void __fastcall FileMenuExitClick(TObject *Sender);
   void __fastcall FormKeyDown(TObject *Sender, WORD &Key, TShiftState Shift);
   void __fastcall FormKeyUp(TObject *Sender, WORD &Key, TShiftState Shift);
   void __fastcall LogMenuFreezeDisplayAtErrorClick(TObject *Sender);
   void __fastcall FileMenuSetLogDirectoryClick(TObject *Sender);
   void __fastcall LogMenuLogDirectoryClick(TObject *Sender);
   void __fastcall HelpMenuHelpClick(TObject *Sender);
   void __fastcall HelpMenuAboutClick(TObject *Sender);
   void __fastcall Button1ImageMouseDown(TObject *Sender,
          TMouseButton Button, TShiftState Shift, int X, int Y);
   void __fastcall Button1ImageMouseUp(TObject *Sender,
          TMouseButton Button, TShiftState Shift, int X, int Y);
   void __fastcall Button2ImageMouseDown(TObject *Sender,
          TMouseButton Button, TShiftState Shift, int X, int Y);
   void __fastcall Button2ImageMouseUp(TObject *Sender,
          TMouseButton Button, TShiftState Shift, int X, int Y);
   void __fastcall Button3ImageMouseDown(TObject *Sender,
          TMouseButton Button, TShiftState Shift, int X, int Y);
   void __fastcall Button3ImageMouseUp(TObject *Sender,
          TMouseButton Button, TShiftState Shift, int X, int Y);
   void __fastcall Button0ImageMouseDown(TObject *Sender,
          TMouseButton Button, TShiftState Shift, int X, int Y);
   void __fastcall Button4ImageMouseDown(TObject *Sender,
          TMouseButton Button, TShiftState Shift, int X, int Y);
   void __fastcall Button4ImageMouseUp(TObject *Sender,
          TMouseButton Button, TShiftState Shift, int X, int Y);
   void __fastcall Button5ImageMouseDown(TObject *Sender,
          TMouseButton Button, TShiftState Shift, int X, int Y);
   void __fastcall Button5ImageMouseUp(TObject *Sender,
          TMouseButton Button, TShiftState Shift, int X, int Y);
   void __fastcall Button6ImageMouseDown(TObject *Sender,
          TMouseButton Button, TShiftState Shift, int X, int Y);
   void __fastcall Button6ImageMouseUp(TObject *Sender,
          TMouseButton Button, TShiftState Shift, int X, int Y);
   void __fastcall Button7ImageMouseDown(TObject *Sender,
          TMouseButton Button, TShiftState Shift, int X, int Y);
   void __fastcall Button7ImageMouseUp(TObject *Sender,
          TMouseButton Button, TShiftState Shift, int X, int Y);
   void __fastcall Button8ImageMouseDown(TObject *Sender,
          TMouseButton Button, TShiftState Shift, int X, int Y);
   void __fastcall Button8ImageMouseUp(TObject *Sender,
          TMouseButton Button, TShiftState Shift, int X, int Y);
   void __fastcall Button9ImageMouseDown(TObject *Sender,
          TMouseButton Button, TShiftState Shift, int X, int Y);
   void __fastcall Button9ImageMouseUp(TObject *Sender,
          TMouseButton Button, TShiftState Shift, int X, int Y);
   void __fastcall ButtonPoundImageMouseDown(TObject *Sender,
          TMouseButton Button, TShiftState Shift, int X, int Y);
   void __fastcall ButtonPoundImageMouseUp(TObject *Sender,
          TMouseButton Button, TShiftState Shift, int X, int Y);
   void __fastcall ButtonStarImageMouseDown(TObject *Sender,
          TMouseButton Button, TShiftState Shift, int X, int Y);
   void __fastcall ButtonStarImageMouseUp(TObject *Sender,
          TMouseButton Button, TShiftState Shift, int X, int Y);
   void __fastcall Button0ImageMouseUp(TObject *Sender,
          TMouseButton Button, TShiftState Shift, int X, int Y);
   void __fastcall ResumeImageMouseDown(TObject *Sender,
          TMouseButton Button, TShiftState Shift, int X, int Y);
   void __fastcall ShowLogImageMouseDown(TObject *Sender,
          TMouseButton Button, TShiftState Shift, int X, int Y);
   void __fastcall ShowLogImageMouseUp(TObject *Sender,
          TMouseButton Button, TShiftState Shift, int X, int Y);
   void __fastcall PauseImageMouseUp(TObject *Sender, TMouseButton Button,
          TShiftState Shift, int X, int Y);
   void __fastcall StopRecordingImageMouseDown(TObject *Sender,
          TMouseButton Button, TShiftState Shift, int X, int Y);
   void __fastcall StopRecordingImageMouseUp(TObject *Sender,
          TMouseButton Button, TShiftState Shift, int X, int Y);
   void __fastcall PauseImageMouseDown(TObject *Sender,
          TMouseButton Button, TShiftState Shift, int X, int Y);
   void __fastcall ResumeImageMouseUp(TObject *Sender, TMouseButton Button,
          TShiftState Shift, int X, int Y);
   void __fastcall CallSystemImageMouseDown(TObject *Sender,
          TMouseButton Button, TShiftState Shift, int X, int Y);
   void __fastcall CallSystemImageMouseUp(TObject *Sender,
          TMouseButton Button, TShiftState Shift, int X, int Y);
   void __fastcall HangupImageMouseDown(TObject *Sender,
          TMouseButton Button, TShiftState Shift, int X, int Y);
   void __fastcall HangupImageMouseUp(TObject *Sender, TMouseButton Button,
          TShiftState Shift, int X, int Y);
   void __fastcall NextStepImageMouseUp(TObject *Sender,
          TMouseButton Button, TShiftState Shift, int X, int Y);
   void __fastcall NextStepImageMouseDown(TObject *Sender,
          TMouseButton Button, TShiftState Shift, int X, int Y);
   void __fastcall SimulationMenuPauseClick(TObject *Sender);
   void __fastcall BargeInImageMouseDown(TObject *Sender,
          TMouseButton Button, TShiftState Shift, int X, int Y);
   void __fastcall BargeInImageMouseUp(TObject *Sender,
          TMouseButton Button, TShiftState Shift, int X, int Y);

   private:	// User declarations

      CommunicationHub<TFServerCommunicationPacket::TF_DESTINATION_ID> commHub;
      TFServerCommunicationDestination TFServerMessages;
      TFSimulatorCommunicationDestination TFSimulatorMessages;
      TCPClientManager* tcpClientManager;
      TFApplicationsManager* applicationsManager;
      MonitorParameters monitorParams;
      CallSystemParameters callSystemParams;
      unsigned int callSystemID;
      MCIDEVICEID mciDevice;
      HWAVEIN hwi;
      DWORD shortAudioPrompts;
      FileLog sysLog;
      TFServerRegistrationInfo regInfo;
      TFApplication* currentApplication;
      int currentAppID;
      Mutex loadingMutex;
      HANDLE hResume;
      HINSTANCE boardLibrary;

      bool onHook;                             // Relates to handset and not board
      bool readyForCall;
      bool connected;
      int testcounter;
      char lastStatus;

		void __fastcall ProcessMCINotify(TMessage& Message);
      void __fastcall ProcessWIMData(TMessage& message);
      void __fastcall Start(void);
      void __fastcall End(void);
      void __fastcall ReadInParameters(void);
      void __fastcall ReadTFSimulatorRegistry(void);
      void __fastcall WriteRegistry(void);
      void __fastcall LoadJPGFromResource(TImage* image, const char* jpgName);
      void __fastcall LoadGraphics(void);
      void __fastcall ResetGraphics(void);
      void __fastcall InitialiseGraphics(void);
      void __fastcall Pause(void);
      void __fastcall Resume(void);
      void __fastcall PrepareNewApplication(void);

   public:		// User declarations

      __fastcall TFormSimulator(TComponent* Owner);
      __fastcall ~TFormSimulator(void);

   	virtual void __fastcall Dispatch(void *Message);
      void __fastcall ProcessOnHook(void);
      void __fastcall ProcessOffHook(void);
      void __fastcall ProcessReadyForCall(bool on);
      void __fastcall ProcessRefresh(char status, int stepNumber, const char* stepName, const char* TAMName);
      void __fastcall ProcessOutgoingCall(void);
      void __fastcall SetHWI(HWAVEIN hwi);
      void __fastcall SetMCIDevice(LONG mciDevice);
      void __fastcall ShowStopRecordingButton(bool on);
      void __fastcall ShowBargeInButton(bool on);
      void __fastcall EnableBargeInButton(bool enabled);
      void __fastcall TerminateEngine(int appID);
      void __fastcall AddLine(const char* line);
      void __fastcall SendEvent(BoardSimulatorCommunicationPacket::BRD_SIMULATOR_EVENT event, int eventInfo = 0);
      void __fastcall SendDigit(char digit);
      void __fastcall PrintToMessageWindow(const char* message);
      void __fastcall Setup(void);
      void __fastcall Cleanup(void);
};

//---------------------------------------------------------------------------
extern PACKAGE TFormSimulator *FormSimulator;
//---------------------------------------------------------------------------
#endif
