#include <stdarg.h>
#include "TIME.H"
#include "stdio.h"
#include "stdlib.h"
#include <ctype.h>
#include <string.h>
#include <map>

#include "StringSupport.h"

using std::string;
using std::vector;
using std::map;

static char formatTmp[1000];

const char *FormattedString( const char* fmt, ... )
{
    va_list arglist;
    va_start( arglist, fmt );

	_vsnprintf( formatTmp, sizeof( formatTmp ), fmt, arglist );

	va_end( arglist );

	return (const char *)&formatTmp[0];
}

void RemovePadding( string &input, char padding )
{
	if ( !input.length() )
	{
		return;
	}
	string tmp = "";
	unsigned int curChar = 0;
	for ( curChar = 0; ( ( curChar < strlen( input.c_str() ) ) && ( input.c_str()[curChar] == padding ) ); curChar++ );
	tmp = &input.c_str()[curChar];
	for ( curChar = strlen( tmp.c_str() ) - 1; ( tmp.c_str()[curChar] == padding ); curChar-- );
   tmp = tmp.substr( 0, curChar+1 );
	input = tmp.c_str();
}

int Tokenize( string input, const char *separator, vector<string> *tokens, bool removeSpaces, int maxTokens )
{
	tokens->clear();
	char *tracer = (char*)input.c_str();
	while ( ( tracer ) && ( strlen( tracer ) ) )
	{
		char *pos = strstr( tracer, separator );
		if ( ( maxTokens != -1 ) && ( (unsigned int)maxTokens == ( tokens->size() + 1 ) ) )
		{
			pos = NULL;
		}
		if ( pos )
		{
         string tmp = SubstringUpTo( tracer, separator );
			tokens->push_back( tmp );
			tracer = pos + strlen( separator );
		}
		else
		{
			tokens->push_back( tracer );
			tracer = NULL;
		}
	}
	if ( removeSpaces )
	{
		for ( unsigned int index = 0; index < tokens->size(); index++ )
		{
			string content = tokens->at( index );
			RemovePadding( content );
			tokens->at( index ) = content;
		}
	}
	return tokens->size();
}

void SearchAndReplace( string& source, const string& toReplace, const string& replaceWith )
{
	if ( ( replaceWith.length() ) && ( strstr( toReplace.c_str(), replaceWith.c_str() ) ) )
	{
		SearchAndReplace( source, toReplace, "\1" );
		SearchAndReplace( source, "\1", replaceWith );
	}
	else
	{
		size_t pos = string::npos;
		while ( ( pos = source.find( toReplace ) ) != std::string::npos )
      {
			source.replace( pos, toReplace.size(), replaceWith );
      }
	}
}

std::string SubstringUpTo( const std::string source, const std::string delimitorString )
{
   size_t pos = string::npos;
   if ( ( pos = source.find( delimitorString ) ) != std::string::npos )
   {
      return( source.substr( 0, pos ) );
   }
   else
   {
      return( "" );
   }
}

unsigned char NormalizeChar( int input, unsigned char normalizeFromChar, unsigned char normalizeToChar )
{
	while ( input > normalizeToChar )
	{
		input -= ( normalizeToChar - normalizeFromChar );
	}
	while ( input < normalizeFromChar )
	{
		input += ( normalizeToChar - normalizeFromChar );
	}
	return (unsigned char)input;
}

string GenerateUniqueString( unsigned int length, char useLower, char useUpper )
{
	static int seedCount;
	srand( (unsigned)time( NULL ) + seedCount );
	seedCount++;
	string dReturn = "";
	time_t dTime;
	time( &dTime );
	tm *dtmtime = localtime( &dTime );
	if ( length <= sizeof( tm ) )
	{
		for ( unsigned int index = 0; ( ( index < sizeof( tm ) ) && ( length > dReturn.length() ) ); index++ )
		{
			char dChar = (char)( *( (char*)(dtmtime) + index ) + rand() );
			dReturn += NormalizeChar( dChar, useLower, useUpper );
		}
	}
	return dReturn;
}

void LowercaseString( string& source )
{
	for( size_t index = 0; index < source.size(); index++ )
   {
		source[index] = (char)tolower( source[index] );
   }
}


string EscapeInvalidURIChars( string strSource )
{
   string strResult;

   if(strSource.empty())
   {
      return(strResult);
   }

   const char DEC2HEX[16 + 1] = "0123456789ABCDEF";
   int pos = 0;
   int srcLen = strSource.length();

   // RFC 3986 section 2.2 Reserved Characters (January 2005)
   const string RESERVED_CHARS = "!*'();:@&=+$,/?#[]";
   // RFC 3986 section 2.3 Unreserved Characters (January 2005)   
   const string UNRESERVED_CHARS = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-_.~";
   // Other characters in a URI must be percent encoded.

   if((strSource.find("%20") != string::npos) || (strSource.find("%25") != string::npos))
   {
      // Source string has already been encoded once
      strResult = strSource;
      return(strResult);
   }

   for(pos = 0; pos < srcLen; pos++)
   {
      if((RESERVED_CHARS.find(strSource[pos]) == string::npos) && (UNRESERVED_CHARS.find(strSource[pos]) == string::npos))
      {
         // Replace this character with percent encoded hex
         strResult += '%';
         strResult += DEC2HEX[strSource[pos] >> 4];
         strResult += DEC2HEX[strSource[pos] & 0x0F];
      }
      else
      {
         strResult += strSource[pos];
      }
   }

   return(strResult);
}


string EscapeInvalidXMLChars( string strSource )
{
   string strDest = "";

   if(strSource.empty())   // Source is empty? Nothing to do.
   {
      return(strDest);
   }

   map<char, string> transformation;
   map<char, string>::iterator ti;
   // string reserved_chars = " & ' \" > < ";
   size_t pos = string::npos;
   size_t sourceLen = strSource.length();

   // Map special character to its transformed value
   transformation['&']  = std::string("&amp;");
   transformation['\''] = std::string("&apos;");
   transformation['"']  = std::string("&quot;");
   transformation['>']  = std::string("&gt;");
   transformation['<']  = std::string("&lt;");

   // OPTIONAL: Create a single string of the reserved chars from contents of transformation map
   // Can use find_first_of() to locate a reserved character; eg: strSource.find_first_of(reserved_chars, pos)
   //    string reserved_chars = "";
   //    for (ti = transformation.begin(); ti != transformation.end(); ti++)
   //    {
   //       reserved_chars += ti->first;
   //    }

   // Verify that the input string hasn't been transformed already
   ti = transformation.begin();
   while((ti != transformation.end()) && ((pos = strSource.find(ti->second)) == string::npos))
   {
      ti++;
   }

   if(pos != string::npos)
   {
      strDest = strSource;
      return(strDest);
   }

   // Copy each character from strSource to strDest, transforming as necessary
   for(pos = 0; pos < sourceLen; pos++)
   {
      ti = transformation.find(strSource[pos]);
      if(ti != transformation.end())
      {
         strDest += transformation[strSource[pos]];
      }
      else
      {
         strDest += strSource[pos];
      }
   }

   return(strDest);
}
// EscapeInvalidXMLChars
