/* -------------------------------------------------------------------------
   ------------------------------------------------------------------------- */
#include <windows.h>
#include <stdio.h>	// sprintf

#include "file_fns.h"

int ChangeDirectory(const char* path)
{
   #ifdef _MSC_VER
      return _chdir(path);
   #else
      return chdir(path);
   #endif
}


/* -------------------------------------------------------------------------
   Name:        DeleteTree

   Purpose:     Deletes all files and directories in the specified path.
   ------------------------------------------------------------------------- */
bool DeleteTree (const char *pszPath)
{
   bool             bOkay;
   bool             bFound;
   HANDLE           hSearch;
   WIN32_FIND_DATA  findInfo;
   char             szMask[_MAX_PATH];
   DWORD            rootAttrib;

   if ((rootAttrib = GetFileAttributes (pszPath)) == 0xFFFFFFFF)
      return false;

   if (!(rootAttrib & FILE_ATTRIBUTE_DIRECTORY))
      return false;

   sprintf (szMask, "%s\\*.*", pszPath);
   hSearch = FindFirstFile (szMask, &findInfo);

   bOkay  = true;
   bFound = true;
   while (hSearch && bFound && bOkay) {
      if (strcmp (findInfo.cFileName, ".")  &&
          strcmp (findInfo.cFileName, ".."))
      {
         if ((findInfo.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY))
         {
            sprintf (szMask, "%s\\%s", pszPath, findInfo.cFileName);
            bOkay = DeleteTree (szMask);
            RemoveDirectory (szMask);

         } else {
            sprintf (szMask, "%s\\%s", pszPath, findInfo.cFileName);
            DeleteFile (szMask);
         }
      }
      bFound = FindNextFile (hSearch, &findInfo);
   }
   FindClose (hSearch);

   if (strcmp (pszPath, ".") && strcmp (pszPath, ".."))
      RemoveDirectory (pszPath);

   return bOkay;
}
//DeleteTree

bool FileExists(const char *pszPath)
{
   FILE     *fp;         // File pointer to check validity of pszPath

   if(pszPath == NULL)
   {
      return(false);
   }
   fp = fopen(pszPath, "r");
   if (fp == NULL)
   {
      return(false);
   }
   fclose(fp);
   return(true);
}
//FileExists

int FindFirst(const char* path, FindData* findData)
{
   #ifdef _MSC_VER
      findData->handle = _findfirst(path, &findData->data);
      return findData->handle == -1 ? -1 : 0;
   #else
      return findfirst(path, findData, FA_NORMAL);
   #endif
}

int FindNext(FindData* findData)
{
   #ifdef _MSC_VER
      return _findnext(findData->handle, &findData->data);
   #else
      return findnext(findData);
   #endif
}

char* GetName(FindData* findData)
{
   #ifdef _MSC_VER
      return findData->data.name;
   #else
      return findData->ff_name;
   #endif
}

void SplitPath(const char* path, char* drive, char* directory, char* name, char* extension)
{
   #ifdef _MSC_VER
      _splitpath(path, drive, directory, name, extension);
   #else
      fnsplit(path, drive, directory, name, extension);
   #endif
}
