#include "HTTPRequest.h"

HTTPRequest::HTTPRequest(void)
{
   // Put in space separators and field separator for request line, and header separator
   // Method, URI, and version are empty strings
   text = "  \r\n\r\n";
   SetMethod("GET");
   SetVersion("HTTP/1.0");
   SetField("User-Agent", "TeleFlow");
   SetField("Accept", "*/*");
}

HTTPRequest::~HTTPRequest(void)
{
}

const char* HTTPRequest::GetMethod(void)
{
   string::size_type index = text.find(' ');

   if(index == string::npos)
   {
      return("");
   }

   method = text.substr(0, index);

   return(method.c_str());
}

const char* HTTPRequest::GetURI(void)
{
   string::size_type index1;
   string::size_type index2;

   if((index1 = text.find(' ')) == string::npos)
   {
      return("");
   }

   index1++;

   if((index2 = text.find(' ', index1)) == string::npos)
   {
      uri = text.substr(index1);
   }
   else
   {
      uri = text.substr(index1, index2);
   }

   return(uri.c_str());
}

const char* HTTPRequest::GetVersion(void)
{
   string::size_type index1;
   string::size_type index2;

   if((index1 = text.find(' ')) == string::npos)
   {
      return("");
   }

   index1++;

   if((index1 = text.find(' ', index1)) == string::npos)
   {
      return("");
   }

   index1++;

   if((index2 = text.substr(index1).find("\r\n")) == string::npos)
   {
      version = text.substr(index1);
   }
   else
   {
      version = text.substr(index1, index2);
   }

   return(version.c_str());
}

void HTTPRequest::SetMethod(const char* text)
{
   string::size_type methodEnd;
   string::size_type endOfRequestLine = this->text.find("\r\n");

   if(endOfRequestLine == string::npos)
   {
      endOfRequestLine = this->text.length();
      this->text += (string) "\r\n";
   }

   if(text == NULL)
   {
      text = "";
   }

   // Find end of current method
   if((methodEnd = this->text.substr(0, endOfRequestLine).find(' ')) == string::npos)
   {
      // Add missing spaces
      this->text.insert(endOfRequestLine, "  ");
      endOfRequestLine += 2;
      methodEnd = endOfRequestLine - 2;
   }

   this->text.replace(this->text.begin(), this->text.begin() + methodEnd, text);
}

void HTTPRequest::SetMethod(HTTPRequest::REQUEST_METHOD method)
{
   switch(method)
   {
      case HTTP_GET: SetMethod("GET"); break;
      case HTTP_POST: SetMethod("POST"); break;
      case HTTP_PUT: SetMethod("PUT"); break;
      case HTTP_DELETE: SetMethod("DELETE"); break;
      case HTTP_OPTIONS: SetMethod("OPTIONS"); break;
      case HTTP_HEAD: SetMethod("HEAD"); break;
      case HTTP_TRACE: SetMethod("TRACE"); break;
   }
}

void HTTPRequest::SetURI(const char* text)
{
   string::size_type uriBegin;
   string::size_type uriEnd;
   string::size_type endOfRequestLine = this->text.find("\r\n");

   if(endOfRequestLine == string::npos)
   {
      endOfRequestLine = this->text.length();
      this->text += (string) "\r\n";
   }

   if(text == NULL)
   {
      text = "";
   }

   // Find beginning of URI
   if((uriBegin = this->text.substr(0, endOfRequestLine).find(' ')) == string::npos)
   {
      this->text.insert(endOfRequestLine, "  ");
      endOfRequestLine += 2;
      uriBegin = endOfRequestLine - 2;
   }
   else
   {
      uriBegin++;
   }

   // Find end of URI
   if((uriEnd = this->text.substr(uriBegin, endOfRequestLine).find(' ')) == string::npos)
   {
      this->text.insert(endOfRequestLine, " ");
      endOfRequestLine += 1;
      uriEnd = endOfRequestLine - 1;
   }
   else
   {
      uriEnd += uriBegin;
   }

   this->text.replace(this->text.begin() + uriBegin, this->text.begin() + uriEnd, text);
}

void HTTPRequest::SetVersion(const char* text)
{
   string::size_type uriBegin;
   string::size_type versionBegin;
   string::size_type endOfRequestLine = this->text.find("\r\n");

   if(endOfRequestLine == string::npos)
   {
      endOfRequestLine = this->text.length();
      this->text += (string) "\r\n";
   }

   if(text == NULL)
   {
      text = "";
   }

   // Find beginning of URI
   if((uriBegin = this->text.substr(0, endOfRequestLine).find(' ')) == string::npos)
   {
      this->text.insert(endOfRequestLine, "  ");
      endOfRequestLine += 2;
      uriBegin = endOfRequestLine - 2;
   }
   else
   {
      uriBegin++;
   }

   // Find beginning of Version
   if((versionBegin = this->text.substr(uriBegin, endOfRequestLine).find(' ')) == string::npos)
   {
      this->text.insert(endOfRequestLine, " ");
      endOfRequestLine += 1;
      versionBegin = endOfRequestLine - 1;
   }
   else
   {
      versionBegin += uriBegin + 1;
   }

   this->text.replace(this->text.begin() + versionBegin, this->text.begin() + endOfRequestLine, text);
}


