#ifndef __HTTPREQUEST_H__
#define __HTTPREQUEST_H__

#ifdef __DLL__
#define HTTPREQUESTAPI __declspec(dllexport)
#else
#define HTTPREQUESTAPI __declspec(dllimport)
#endif

#include "InternetMessage.h"

// Class for managing HTTP requests as per RFC 1945
class HTTPREQUESTAPI HTTPRequest : public InternetMessage
{
   public:

      enum REQUEST_METHOD
      {
         HTTP_GET = 0,
         HTTP_POST,
         HTTP_PUT,
         HTTP_DELETE,
         HTTP_OPTIONS,
         HTTP_HEAD,
         HTTP_TRACE
      };

      // The first line of an HTTP request header has this format:
      // "MMMMM UUUUUUUU TTTT/V.V\r\n"
      // where MMMMM is the HTTP request method, UUUUUUUU is the HTTP request URI,
      // TTTT is the internet message type, and V.V is the version e.g. HTTP/1.0

      // Sets the HTTP request method to GET
      // Sets the internet message type and version to HTTP/1.0
      // Sets the User-Agent header field to "TeleFlow"
      // Sets the Accept header field to "*/*"
      HTTPRequest(void);

      ~HTTPRequest(void);

      // Returns the HTTP request method
      // Returns an empty string if the method is not found
      virtual const char* GetMethod(void);

      // Returns the HTTP request URI
      // Returns an empty string if the URI is not found
      virtual const char* GetURI(void);

      // Returns the internet message type and version
      // Returns an empty string if the version is not found
      virtual const char* GetVersion(void);

      // Sets the HTTP request method to text
      // Clears the HTTP request method if text is NULL
      virtual void SetMethod(const char* text);

      // Sets the HTTP request method to method
      // Has no effect if method is an invalid REQUEST_METHOD
      virtual void SetMethod(HTTPRequest::REQUEST_METHOD method);

      // Sets the HTTP request URI to text
      // Clears the HTTP request URI if text is NULL
      virtual void SetURI(const char* text);

      // Sets the internet message type and version to text
      // Clears the internet message type and version if text is NULL
      virtual void SetVersion(const char* text);

   protected:

      string method;
      string uri;
      string version;
};

#endif

