#ifndef __HTTPRESPONSE_H__
#define __HTTPRESPONSE_H__

#ifdef __DLL__
#define HTTPRESPONSEAPI __declspec(dllexport)
#else
#define HTTPRESPONSEAPI __declspec(dllimport)
#endif

#include <string>
using std::string;

#include "InternetMessage.h"

// Class for managing HTTP responses as per RFC 1945
class HTTPRESPONSEAPI HTTPResponse : public InternetMessage
{
   public:

      enum STATUS_CODE
      {
         INVALID_STATUS = -1
      };

      // Format: "TTTT/V.V XXX MMMMMMMM\r\n..."
      // where TTTT is the internet message type and V.V is the version e.g. HTTP/1.0,
      // XXX is the status code, and MMMMMMMM is the reason phrase

      HTTPResponse(void);
      virtual ~HTTPResponse(void);

      // Returns the reason phrase that follows the internet message type, version,
      // and status code in the HTTP response header
      virtual const char* GetReasonPhrase(void);

      // Returns the status code that follows the internet message type and version
      // in the HTTP response header
      // Returns INVALID_STATUS if the status code is not present, or the status
      // code contains non-numeric data
      virtual HTTPResponse::STATUS_CODE GetStatusCode(void) const;

      // Returns true if the complete message has been received
      // Returns false if a header separator "\r\n\r\n" is not present
      // Returns false if the message is shorter than required by a Content-Range field
      // Returns false if the message is shorter than required by a Content-Length field
      virtual bool HaveCompleteMessage(unsigned int *receiveContentLength = NULL) const;

   protected:

      string reasonPhrase;
};

#endif

