#include <stdio.h>                  // sprintf

#include "..\escutil\iv_str.h"      // FindCaseInsensitiveString
#include "HTTPTransaction.h"

HTTPTransaction::HTTPTransaction(Log* outLog, const char* url, bool persistant) : InternetMessageTCPTransaction(outLog, persistant)
{
   request = new HTTPRequest;
   response = new HTTPResponse;
   SetURL(url);
}

HTTPTransaction::HTTPTransaction(Log* outLog, ACE_HANDLE handle, const char* url) : InternetMessageTCPTransaction(outLog, handle)
{
   request = new HTTPRequest;
   response = new HTTPResponse;
   SetURL(url);
}

HTTPTransaction::~HTTPTransaction(void)
{
   delete request;
   delete response;
}

const char* HTTPTransaction::GetURL(void)
{
   return(url.c_str());
}

HTTPRequest* HTTPTransaction::Request(void)
{
   return((HTTPRequest*) request);
}

HTTPResponse* HTTPTransaction::Response(void)
{
   return((HTTPResponse*) response);
}

void HTTPTransaction::SetPeerName(const char* name)
{
   char number[14];
   InternetMessageTCPTransaction::SetPeerName(name);
   sprintf(number, "%d", peerPort);
   Request()->SetField("Host", (peerName + (string) ":" + (string) number).c_str());
   ConstructURL();
}

void HTTPTransaction::SetPeerPort(unsigned short port)
{
   char number[14];
   InternetMessageTCPTransaction::SetPeerPort(port);
   sprintf(number, "%d", peerPort);
   Request()->SetField("Host", (peerName + (string) ":" + (string) number).c_str());
   ConstructURL();
}

void HTTPTransaction::SetURL(const char* url)
{
   string::size_type index;
   char number[14];

   this->url.clear();
   peerName.clear();
   peerPort = 80;

   if(url == NULL)
   {
      return;
   }

   this->url = url;

   if(this->url.substr(0, 7) == "http://")
   {
      peerName = this->url.substr(7);
      DisableSSL();
   }
   else if(this->url.substr(0, 5) == "http:")
   {
      peerName = this->url.substr(5);
      DisableSSL();
   }
   else if(this->url.substr(0, 8) == "https://")
   {
      peerPort = 443;
      peerName = this->url.substr(8);
      EnableSSL();
   }
   else if(this->url.substr(0, 6) == "https:")
   {
      peerPort = 443;
      peerName = this->url.substr(6);
      EnableSSL();
   }
   else
   {
      peerName = this->url;
      DisableSSL();
   }

   if((index = peerName.find('/')) != string::npos)
   {
      Request()->SetURI(peerName.substr(index).c_str());
      peerName = peerName.substr(0, index);
   }
   else
   {
      Request()->SetURI("/");
   }

   if((index = peerName.find(':')) != string::npos)
   {
      peerPort = (unsigned short) atoi(peerName.substr(index + 1).c_str());
      peerName = peerName.substr(0, index);
   }

   sprintf(number, "%d", peerPort);
   Request()->SetField("Host", (peerName + (string) ":" + (string) number).c_str());
}

void HTTPTransaction::ConstructURL(void)
{
   char number[14];
   url = (string) "http" + (string) (IsSSLEnabled() ? "s" : "") + (string) "://";
   url += peerName;
   url += ":";
   sprintf(number, "%d", peerPort);
   url += number;
   url += (string) Request()->GetURI();
}

