/* -------------------------------------------------------------------------
   File:        INET_CODE.H

   Notes:       - Name mangling has been turned off.
                - Function names are prefixed with an underscore.
   ------------------------------------------------------------------------- */
#ifndef INET_CODE_H
#ifndef _INET_CODE_H
#define INET_CODE_H

#include <winsock2.h>
#include <windows.h>

/* Winsock Definitions */
#define WINSOCK_HI       2
#define WINSOCK_LO       0

#ifdef __DLL__
#define MYNETAPI extern "C" __declspec(dllexport)
#else
#define MYNETAPI extern "C" __declspec(dllimport)
#endif

#define NET_EVT_NULL          0
#define NET_EVT_CLOSED        1
#define NET_EVT_OPEN          2
#define NET_EVT_READ          3
#define NET_EVT_WRITE         4
#define NET_EVT_ACCEPT        5

enum NET_STATUS { NET_CLOSED, NET_CONNECTING, NET_CONNECTED, NET_ERROR };

typedef struct
{
   SOCKET hSocket;
   HANDLE hEvent;
} NET_STORAGE;

MYNETAPI void PrintErrorMsg(char* msg, int errorCode);
MYNETAPI const char* WSAErrorMsg(int errorCode);
// *********************** COMMON FUNCTIONS ***********************************
MYNETAPI bool NetCloseSocket(NET_STORAGE* p);
MYNETAPI int NetSend(NET_STORAGE* p, const char* buf, int bufSize);
MYNETAPI int NetRecv(NET_STORAGE* p, void* buf, int bufSize);
MYNETAPI bool NetGetEvent(NET_STORAGE* p, int& event);

// ******************** STARTUP / TEARDOWN FUNCTIONS **************************
MYNETAPI bool NetInitialize(void);
MYNETAPI void NetShutdown(void);

// ************************ SERVER FUNCTIONS **********************************
MYNETAPI bool NetListen(NET_STORAGE* p, int port);
MYNETAPI bool NetAccept(NET_STORAGE* p, NET_STORAGE* np, HANDLE hEvent);

// ************************ CLIENT FUNCTIONS **********************************
MYNETAPI bool NetConnect(NET_STORAGE* p, const char* pszHost, int port,
   long lNetworkEvents = FD_CONNECT);
MYNETAPI bool NetSendConnect(NET_STORAGE* p, const char* pszHost, int port);

#endif                          
#endif
