/* -------------------------------------------------------------------------
   Copyright (c) 2002 enGenic.  All rights reserved.
   ------------------------------------------------------------------------- */
#include "xmllist.h"
#include "..\3rdParty\tinyxml\tinystr.h"
#include "..\escutil\VarController.h"
/* *************************************************************************
   *************************  XMLList Class  *******************************
   ************************************************************************* */

/* -------------------------------------------------------------------------
   Name:        XMLList::XMLList                               (Constructor)
   ------------------------------------------------------------------------- */
XMLList::XMLList(Log* outLog) : Head()
{
   ivLog = outLog;
}
/*XMLList::XMLList*/


/* -------------------------------------------------------------------------
   Name:        XMLList::~XMLList                               (Destructor)
   ------------------------------------------------------------------------- */
XMLList::~XMLList(void)
{
   // Nothing required yet
}
/*XMLList::~XMLList*/


/* -------------------------------------------------------------------------
   Name:        XMLList::FindXML
   ------------------------------------------------------------------------- */
XML* XMLList::FindXML (int refNo)
{
   XML *temp;

   temp = (XML*)First();
   while (temp)
   {
      if (temp->GetReferenceNo () == refNo)
         return temp;
      temp = (XML*)temp->Suc();
   }
   return NULL;
}
/*XMLList::FindXML*/

void XMLList::Reset(void)
{
   Clear();
   ivLog->Print("      XML list cleared\n");
}

/* *************************************************************************
   **************************  XML Class  **********************************
   ************************************************************************* */

/* -------------------------------------------------------------------------
   Name:        XML::XML                                       (Constructor)
   ------------------------------------------------------------------------- */
XML::XML (Log *outLog) : Link()
{
   ivLog      = outLog;
   refNo      = 0;
   xFetchList = new XMLFetchList (ivLog);
   xmlDoc     = new TiXmlDocument();
   xmlPrinter = new TiXmlPrinter();
   // Moved to CopyToBuffer() method; do not Accept until we're ready to print
   // xmlDoc->Accept(xmlPrinter);
}
/*XML::XML*/


/* -------------------------------------------------------------------------
   Name:        XML::~XML                                       (Destructor)
   ------------------------------------------------------------------------- */
XML::~XML()
{
   delete xmlPrinter;
   delete xmlDoc;
   if (xFetchList != NULL)
   {
      delete xFetchList;
   }
}
/*XML::~XML*/


/* -------------------------------------------------------------------------
   Name:        XML::SetReferenceNo                                 (Public)
   ------------------------------------------------------------------------- */
void XML::SetReferenceNo (int value)
{
   refNo = value;
}
/*XML::SetReferenceNo*/


/* -------------------------------------------------------------------------
   Name:        XML::GetReferenceNo                                 (Public)
   ------------------------------------------------------------------------- */
int  XML::GetReferenceNo()
{
   return refNo;
}
/*XML::GetReferenceNo*/


/* -------------------------------------------------------------------------
   Name:        XML::LoadFromBuffer                                 (Public)

   Purpose:     Load XML from a memory buffer

   Modified:    May 13/02 - TF: Created
   ------------------------------------------------------------------------- */
IV_STATUS XML::LoadFromBuffer(const char* pBuf)
{
   // Load buffer into XML object
   // ---------------------------
   xmlDoc->Parse(pBuf);
   if(xmlDoc->Error())
   {
      ivLog->Print("      ERROR: Could not parse XML\n");
      ivLog->Print("         %s\n", xmlDoc->ErrorDesc());
      return IVR_FAIL;
   }

   return IVR_SUCCESS;
}
/*XML::LoadFromBuffer*/


/* -------------------------------------------------------------------------
   Name:        XML::LoadFromFile                                   (Public)

   Purpose:     Load XML from a file on disk

   Modified:    May 13/02 - TF: Created
   ------------------------------------------------------------------------- */
IV_STATUS XML::LoadFromFile(const char* pszFile)
{
   FILE     *pFile;         // File pointer to check validity of pszFile

   // Verify requested file exists
   // ----------------------------
   pFile = fopen(pszFile, "r");
   if (pFile == NULL)
   {
      ivLog->Print ("      ERROR: Could not find file '%s'\n", pszFile);
      return IVR_FAIL;
   }
   fclose(pFile);

   // Load file into XML object
   // -------------------------
   if(!xmlDoc->LoadFile(pszFile))
   {
      ivLog->Print("      ERROR: File '%s' does not contain a valid XML document\n", pszFile);
      ivLog->Print("         %s\n", xmlDoc->ErrorDesc());
      return IVR_FAIL;
   }

   return IVR_SUCCESS;
}
/*XML::LoadFromFile*/


/* -------------------------------------------------------------------------
   Name:        XML::CopyToBuffer                                   (Public)

   Purpose:     Save XML to a character buffer

   Modified:    May 14/02 - TF: Created
   ------------------------------------------------------------------------- */
IV_STATUS XML::CopyToBuffer(char*& pText)
{
   if(!xmlDoc->Accept(xmlPrinter))
   {
      pText = NULL;
      return(IVR_FAIL);
   }
   const char* xmlString = xmlPrinter->CStr();
   pText = new char[strlen(xmlString) + 1];
   if(pText == NULL)
   {
      ivLog->Print ("      ERROR: Out of memory.\n");
      return IVR_FATAL;
   }
   strcpy(pText, xmlString);
   return IVR_SUCCESS;
}
/*XML::CopyToBuffer*/


/* *************************************************************************
   **********************  XMLFetchList Class  *****************************
   ************************************************************************* */

/* -------------------------------------------------------------------------
   Name:        XMLFetchList::XMLFetchList                     (Constructor)
   ------------------------------------------------------------------------- */
XMLFetchList::XMLFetchList (Log *outLog) : Head()
{
   ivLog = outLog;
}
/*XMLFetchList::XMLFetchList*/


/* -------------------------------------------------------------------------
   Name:        XMLFetchList::~XMLFetchList                     (Destructor)
   ------------------------------------------------------------------------- */
XMLFetchList::~XMLFetchList()
{
   // Nothing required yet - TF May 16/02
}
/*XMLFetchList::~XMLFetchList*/


/* -------------------------------------------------------------------------
   Name:        XMLFetchList::FindFetch                             (Public)

   Purpose:     Find an open fetch object
   ------------------------------------------------------------------------- */
XMLFetch* XMLFetchList::FindFetch(const char* pszReqCtxt)
{
   XMLFetch* temp;

   temp = (XMLFetch*)First();
   while (temp)
   {
      if (stricmp(temp->pszContext, pszReqCtxt) == 0)
      {
         // Found!
         return temp;
      }
      temp = (XMLFetch*)temp->Suc();
   }
   // Not found.
   return NULL;
}
/*XMLFetchList::FindFetch*/


/* *************************************************************************
   ************************  XMLFetch Class  *******************************
   ************************************************************************* */

/* -------------------------------------------------------------------------
   Name:        XMLFetch::XMLFetch                             (Constructor)
   ------------------------------------------------------------------------- */
XMLFetch::XMLFetch (Log *outLog, const char *pszNodes) : Link()
{
   ivLog          = outLog;
   pszCurToken    = NULL;
   xCurElement    = NULL;

   pszContext    = new char [strlen(pszNodes)+1];
   if (pszContext != NULL)
   {
      strcpy (pszContext, pszNodes);
      pszCurToken = new char [strlen(pszContext)+1];
      if (pszCurToken != NULL)
         pszCurToken[0] = '\0';
   }
}
/*XMLFetch::XMLFetch*/


/* -------------------------------------------------------------------------
   Name:        XMLFetch::~XMLFetch                             (Destructor)
   ------------------------------------------------------------------------- */
XMLFetch::~XMLFetch()
{
   delete[] pszContext;
   delete[] pszCurToken;
}
/*XMLFetch::~XMLFetch*/


/* -------------------------------------------------------------------------
   Name:        XMLFetch::FindFirst                                 (Public)

   Purpose:     Find the first occurence of the node pointed to by
                'pszContext'.

   Notes:       - Need special handling for first occurence because
                  we must parse the tree until we find the node.
                - Subsequent searches use 'FindNext' and continue under
                  the same parent node.
   ------------------------------------------------------------------------- */
IV_STATUS XMLFetch::FindFirst(TiXmlDocument* xmlDoc)
{
   int           ctxtPos = 0;        // Current position in Context string
   int           tokenPos;           // Current position in token string

   // Check validity of required strings
   if (pszContext == NULL)
   {
      ivLog->Print ("      ERROR: No node specified to fetch from.\n");
      return IVR_FATAL;
   }
   if (pszCurToken == NULL)
   {
      ivLog->Print ("      ERROR: Out of memory.\n");
      return IVR_FATAL;
   }

   ivLog->Print("      Fetching first node...\n");
   xCurElement = NULL;

   do
   {
      // --- Parse out next token
      tokenPos = 0;
      while (pszContext[ctxtPos] != '\0' && (pszContext[ctxtPos] != '-' && pszContext[ctxtPos+1] != '>'))
      {
         pszCurToken[tokenPos] = pszContext[ctxtPos];
         ctxtPos++;
         tokenPos++;
      }
      pszCurToken[tokenPos] = '\0';
      // Assertion: have a complete token in pszCurToken

      if (pszContext[ctxtPos] != '\0')
      {
         // Skip the special TeleFlow characters '->'
         ctxtPos = ctxtPos + 2;
      }

      // Moving down a level in the XML document tree...
      xCurElement = xCurElement == NULL ? xmlDoc->RootElement() : xCurElement->FirstChildElement();
      ivLog->Print("         Current token sought = '%s'\n", pszCurToken);  // @@@
      ivLog->Print("         ValueTStr = '%s'\n", xCurElement == NULL ? "NULL" : xCurElement->ValueTStr().c_str());  // @@@

      // --- Find next node
      while((xCurElement != NULL) && (xCurElement->ValueTStr() != pszCurToken))
      {
         // Keep searching
         xCurElement = xCurElement->NextSiblingElement();
         ivLog->Print("         ValueTStr = '%s'\n", xCurElement == NULL ? "NULL" : xCurElement->ValueTStr().c_str());  // @@@
      }

      if (xCurElement == NULL)
      {
         ivLog->Print("         XML Node '%s' not found\n", pszCurToken);
         return IVR_FAIL;
      }
   } while(pszContext[ctxtPos] != '\0');

   return IVR_SUCCESS;
}
/*XMLFetch::FindFirst*/


/* -------------------------------------------------------------------------
   Name:        XMLFetch::FindNext                                  (Public)

   Purpose:     Find the next occurrence of the node pointed to by
                'pszContext'.

   Modified:    May 17/02 - TF: Created

   Notes:       -  Assumes 'FindFirst' has already been called to
                   parse the XML document tree.

                -  'FindNext' only searches for subsequent occurrences of
                   the node under a single parent node.
   ------------------------------------------------------------------------- */
IV_STATUS XMLFetch::FindNext()
{
   ivLog->Print("      Fetching next node...\n");
   ivLog->Print("         Current token sought = '%s'\n", pszCurToken);  // @@@

   // Find next node
   TiXmlElement* nextElement = xCurElement->NextSiblingElement();
   ivLog->Print("         ValueTStr = '%s'\n", nextElement == NULL ? "NULL" : nextElement->ValueTStr().c_str());  // @@@
   while((nextElement != NULL) && (nextElement->ValueTStr() != pszCurToken))
   {
      // Keep searching
      nextElement = nextElement->NextSiblingElement();
      ivLog->Print("         ValueTStr = '%s'\n", nextElement == NULL ? "NULL" : nextElement->ValueTStr().c_str()); // @@@
   }

   if(nextElement != NULL)
   {
      xCurElement = nextElement;
      return IVR_SUCCESS;
   }
   else
   {
      return IVR_FAIL;
   }
}
/*XMLFetch::FindNext*/


/* -------------------------------------------------------------------------
   Name:        XMLFetch::FindPrev                                  (Public)

   Purpose:     Find the previous occurrence of the node pointed to by
                'pszContext'.

   Modified:    Jul.22/02 - TF: Created

   Notes:       -  Assumes 'FindFirst' has already been called to
                   parse the XML document tree.

                -  'FindPrev' only searches for previous occurrences of
                   the node under a single parent node.  If attempt to
                   search beyond that point, IVR_FAIL will be returned.
   ------------------------------------------------------------------------- */
IV_STATUS XMLFetch::FindPrev()
{
   ivLog->Print("      Fetching previous node...\n");
   TiXmlNode* previousElement = xCurElement;
   ivLog->Print("         Current token sought = '%s'\n", pszCurToken);  // @@@

   // Find previous node
   do
   {
      previousElement = previousElement->PreviousSibling();
      while((previousElement != NULL) && (previousElement->Type() != TiXmlNode::ELEMENT))
      {
         // Not an ELEMENT node
         ivLog->Print("      Not an ELEMENT node...\n"); // @@@
         previousElement = previousElement->PreviousSibling();
      }
      ivLog->Print("         ValueTStr = '%s'\n", previousElement == NULL ? "NULL" : previousElement->ValueTStr().c_str());  // @@@
   }
   while((previousElement != NULL) && (previousElement->ValueTStr() != pszCurToken));

   if(previousElement != NULL)
   {
      xCurElement = previousElement->ToElement();
      return IVR_SUCCESS;
   }
   else
   {
      return IVR_FAIL;
   }
}
/*XMLFetch::FindPrev*/


/* -------------------------------------------------------------------------
   Name:        XMLFetch::CaptureData                               (Public)

   Purpose:     Capture the value and attributes of the current node
                of the XML Fetch object into TeleFlow variables

   Modified:    May 17/02 - TF: Created
                Jun 01/02 - TF: Fixed memory leak
                Jun 12/02 - TF: Support '@' symbols within XML data

   Notes:       -  TO DO: Make the handling of missing attributes a user
                   defined setting, ie: ignore (return IVR_SUCCESS) or error
                   (return IVR_FATAL).  Also requires a TFDesigner change
                   to XML Fetch step - TF Jun.6/02
   ------------------------------------------------------------------------- */
IV_STATUS XMLFetch::CaptureData(PIVRINFO pInfo,
                                const char *pszValueVar,
                                const char *pszAttrText,
                                XMLFetch::SCOPE varScope)
{
   size_t         bufsize;             // buffer size
   char*          pszAttrToken = NULL; // attribute to be retrieved
   char*          pszVarToken = NULL;  // variable to save attribute value to
   unsigned int   srcpos;              // current position in source string
   unsigned int   dstpos;              // current position in destination string
   string         xmlString = "";      // @@@
   const char*    xmlText;             // @@@ value retrieved from an XML node
   bool ret;

   if (pszValueVar != NULL && *pszValueVar != '\0')
   {
      // User wants to capture node value to a variable

      // Does the node have a text value?
      if((xCurElement != NULL) && (xCurElement->FirstChild() != NULL))
      {
         if(xCurElement->FirstChild()->Type() == TiXmlNode::TEXT)
         {
            xmlText = xCurElement->FirstChild()->ToText()->Value();
            if (xmlText == NULL)
               xmlText = "";
         }
         else
         {
            // Node does not have a text value - copy entire NODE into a string
            ivLog->Print("      Retrieve NODE as XML\n");
            for (TiXmlNode* pChild = xCurElement->FirstChild(); pChild != 0; pChild = pChild->NextSibling())
            {
               XML::XMLtoString(pChild, xmlString);
            }
            xmlText = xmlString.c_str();
         }
      }
      else
      {
         xmlText = "";
      }

      switch(varScope)
      {
         case XMLFetch::DEFAULT:
         {
            ret = pInfo->variables->Set(pszValueVar, xmlText);
            break;
         }

         case XMLFetch::LOCAL:
         {
            ret = pInfo->variables->SetLocal(pszValueVar, xmlText);
            break;
         }

         case XMLFetch::GLOBAL:
         {
            ret = pInfo->variables->SetGlobal(pszValueVar, xmlText);
            break;
         }

         default:
         {
            ivLog->Print("      ERROR: Invalid XMLFetch::SCOPE value: %d\n", varScope);
            ret = false;
         }
      }

      if(!ret)
      {
         ivLog->Print("      ERROR: Failed to set variable '%s'\n", pszValueVar);
         return(IVR_FATAL);
      }

   } // endif
   // else: User doesn't care about node value

   if (pszAttrText != NULL && *pszAttrText != '\0')
   {
      // User wants to capture attribute value(s) to variable(s)
      bufsize = strlen(pszAttrText)+1;
      pszAttrToken = new char[bufsize];
      memset(pszAttrToken, 0, bufsize);
      pszVarToken = new char[bufsize+1]; // need 1 extra for the '@' symbol
      memset(pszVarToken, 0, bufsize);

      if (pszAttrToken == NULL || pszVarToken == NULL)
      {
         ivLog->Print ("      ERROR: Out of memory.\n");
         return IVR_FATAL;
      }
      srcpos = 0;

      while (pszAttrText[srcpos] != '\0')
      {
         pszAttrToken[0] = '\0';
         pszVarToken[0] = '\0';

         // Verify that the node has attributes - TF May 22/02
			//
			// *** The following statement will crash when the node searched for
         //     is the top-level node.  This problem may need to be fixed
         //     eventually, and is on the tech list.  We are not sure if the
         //     error is ours or in Borland's XML support.  Changes may also
         //     need to be made to the FindFirst() method as this is where
         //     the node xCurNode is created. - TF May 6, 2003.
			//
         if(xCurElement->FirstAttribute() == NULL)
         {
            ivLog->Print("      Warning: Node has no attributes\n");
            delete pszAttrToken;
            delete pszVarToken;
            return IVR_FAIL;
         }

         // Skip white space until we find a token
         while (isspace(pszAttrText[srcpos]) && pszAttrText[srcpos] != '\0')
         {
            srcpos++;
         }

         if (pszAttrText[srcpos] != '\0')
         {
            // Copy the current token as an attribute
            dstpos = 0;
            while (!isspace(pszAttrText[srcpos]) &&
            /***/  pszAttrText[srcpos] != '\0'   && pszAttrText[srcpos] != '\n' )
            {
               pszAttrToken[dstpos] = pszAttrText[srcpos];
               dstpos++;
               srcpos++;
            }
            pszAttrToken[dstpos] = '\0';

            // Skip any white space
            while (isspace(pszAttrText[srcpos]) && pszAttrText[srcpos] != '\0' &&
            /***/  pszAttrText[srcpos] != '\n' )
            {
               srcpos++;
            }

            // Check if next token is "as " or "AS "
            if ( toupper(pszAttrText[srcpos]) == 'A' &&
            /**/ toupper(pszAttrText[srcpos+1]) == 'S' &&
            /**/ isspace(pszAttrText[srcpos+2]) )
            {
               // Found "as", therefore next token is a variable token
               srcpos = srcpos + 3;
               pszVarToken[0] = '@';
               dstpos = 1;
               // Copy the current token as a variable
               while (pszAttrText[srcpos] != '\n' && pszAttrText[srcpos] != '\0')
               {
                  if (!isspace(pszAttrText[srcpos]))
                  {
                     pszVarToken[dstpos] = pszAttrText[srcpos];
                     dstpos++;
                  }
                  srcpos++;
               } // endwhile
               pszVarToken[dstpos] = '\0';
            } else {
               // Did not find "as", therefore there is no variable token
               // Use the uppercase of the attribute token as the variable name
               pszVarToken[0] = '@';
               for (dstpos = 0; dstpos < strlen(pszAttrToken); dstpos++)
               {
                  pszVarToken[1+dstpos] = (char)toupper(pszAttrToken[dstpos]);
               }
               dstpos++;
               pszVarToken[dstpos] = '\0';
            } // endif

            const char* attributeValue  = xCurElement->Attribute(pszAttrToken);
            // Determine if this node has an attribute by this name
            if(attributeValue != NULL)
            {
               // Display TeleFlow variables as we set them
               ivLog->Print ("         %s = (%s) '%s'\n", pszVarToken, pszAttrToken, attributeValue);

               switch(varScope)
               {
                  case XMLFetch::DEFAULT:
                  {
                     ret = pInfo->variables->Set(pszVarToken, attributeValue);
                     break;
                  }

                  case XMLFetch::LOCAL:
                  {
                     ret = pInfo->variables->SetLocal(pszVarToken, attributeValue);
                     break;
                  }

                  case XMLFetch::GLOBAL:
                  {
                     ret = pInfo->variables->SetGlobal(pszVarToken, attributeValue);
                     break;
                  }

                  default:
                  {
                     ivLog->Print("      ERROR: Invalid XMLFetch::SCOPE value: %d\n", varScope);
                     ret = false;
                  }
               }

               if(!ret)
               {
                  ivLog->Print("      ERROR: Failed to set variable: %s\n", pszValueVar);
                  delete[] pszAttrToken;
                  delete[] pszVarToken;
                  return(IVR_FATAL);
               }
            }
            else
            {
               ivLog->Print("      ERROR: No such attribute '%s'\n", pszAttrToken);
               delete[] pszAttrToken;
               delete[] pszVarToken;
               return IVR_FATAL;
               //
               // TO DO: Make the handling of missing attributes a user-defined
               //        setting  - TF Jun.6/02
            }

         } // endif
      } // endwhile

   // else: User doesn't care about attributes
   } // endif

   // --- Cleanup memory used
   delete[] pszAttrToken;
   delete[] pszVarToken;

   return(IVR_SUCCESS);
}
/*XMLFetch::CaptureData*/


void XML::XMLtoString(TiXmlNode* pParent, string &outputStr)
{
   if(pParent == NULL)
      return;

	TiXmlNode* pChild;
   // TiXmlText* pText;
	int type = pParent->Type();
   string nodeName = "";
   string subStr = "";

	switch(type)
	{
	   case TiXmlNode::DOCUMENT:
         // printf( "Document" );
		   break;

   	case TiXmlNode::ELEMENT:
	   	// printf( "Element [%s]", pParent->Value() );
         nodeName = pParent->Value();
         outputStr += "<";
         outputStr += nodeName;
         if(pParent->ToElement() != NULL)
         {
            TiXmlAttribute* pAttrib = pParent->ToElement()->FirstAttribute();
            while(pAttrib)
            {
               outputStr += " ";
               outputStr += pAttrib->Name();
               outputStr += "=\"";
               outputStr += pAttrib->Value();
               outputStr += "=\"";
               pAttrib=pAttrib->Next();
            }
         }
         outputStr += ">" ;
		   break;

	case TiXmlNode::COMMENT:
      // printf( "Comment: [%s]", pParent->Value());
		break;

	case TiXmlNode::UNKNOWN:
      // printf( "Unknown" );
		break;

	case TiXmlNode::TEXT:
      // printf( "Text: [%s]", pText );
      outputStr += pParent->ToText()->Value();
		break;

	case TiXmlNode::DECLARATION:
      // printf( "Declaration" );
		break;

	default:
		break;
	}

   // Now parse and display the children of this node
   subStr = "";
   for (pChild = pParent->FirstChild(); pChild != 0; pChild = pChild->NextSibling())
   {
      XML::XMLtoString(pChild, subStr);
      outputStr += subStr;
   }

   // Add closing tag
   if(type == TiXmlNode::ELEMENT)
   {
      outputStr += "</";
      outputStr += nodeName;
      outputStr += ">\n";
   }
}
// XML::XMLtoString

